@@ -129,7 +129,7 @@ static void align_stack(ChunkStack& cs, int col, bool align_single, log_sev_t se
    if ((cs.Len() > 1) || (align_single && (cs.Len() == 1)))
    {
       LOG_FMT(sev, "%s: max_col=%d\n", __func__, col);
-      while ((pc = cs.Pop()) != NULL)
+      while ((pc = cs.Pop_Back()) != NULL)
       {
          align_to_column(pc, col);
          pc->flags |= PCF_WAS_ALIGNED;
@@ -189,7 +189,7 @@ static void align_add(ChunkStack& cs, chunk_t *pc, int& max_col, int min_pad, bo
       max_col = 0;
    }
 
-   cs.Push(pc);
+   cs.Push_Back(pc);
    if (min_col > max_col)
    {
       max_col = min_col;
@@ -230,6 +230,31 @@ void quick_align_again(void)
 }
 
 
+void quick_indent_again(void)
+{
+   chunk_t *pc;
+   chunk_t *tmp;
+
+   for (pc = chunk_get_head(); pc; pc = chunk_get_next(pc))
+   {
+      if (pc->indent.ref)
+      {
+         tmp = chunk_get_prev(pc);
+         if (chunk_is_newline(tmp))
+         {
+            int col = pc->indent.ref->column + pc->indent.delta;
+
+            indent_to_column(pc, col);
+            LOG_FMT(LINDENTAG, "%s: [%d] indent [%s] to %d based on [%s] @ %d:%d\n",
+                    __func__, pc->orig_line, pc->text(), col,
+                    pc->indent.ref->text(),
+                    pc->indent.ref->orig_line, pc->indent.ref->column);
+         }
+      }
+   }
+}
+
+
 void align_all(void)
 {
    if (cpd.settings[UO_align_typedef_span].n > 0)
@@ -247,11 +272,6 @@ void align_all(void)
       align_oc_msg_colons();
    }
 
-   if (cpd.settings[UO_align_oc_decl_colon].b)
-   {
-      align_oc_decl_colon();
-   }
-
    /* Align variable definitions */
    if ((cpd.settings[UO_align_var_def_span].n > 0) ||
        (cpd.settings[UO_align_var_struct_span].n > 0))
@@ -283,6 +303,12 @@ void align_all(void)
       align_oc_msg_spec(cpd.settings[UO_align_oc_msg_spec_span].n);
    }
 
+   /* Align OC colons */
+   if (cpd.settings[UO_align_oc_decl_colon].b)
+   {
+      align_oc_decl_colon();
+   }
+
    /* Align variable defs in function prototypes */
    if (cpd.settings[UO_align_func_params].b)
    {
@@ -477,7 +503,10 @@ void align_preprocessor(void)
       cur_as = &as;
       if (pc->type == CT_MACRO_FUNC)
       {
-         cur_as = &asf;
+         if (!cpd.settings[UO_align_pp_define_together].b)
+         {
+            cur_as = &asf;
+         }
 
          /* Skip to the close paren */
          pc = chunk_get_next_nc(pc); // point to open (
@@ -804,7 +833,7 @@ static void align_same_func_call_params()
       {
          if (chunk_is_newline(pc))
          {
-            for (idx = 0; idx <= (int)as.size(); idx++)
+            for (idx = 0; idx < (int)as.size(); idx++)
             {
                as[idx].NewLines(pc->nl_count);
             }
@@ -925,6 +954,8 @@ static void align_func_proto(int span)
    LOG_FMT(LALIGN, "%s\n", __func__);
    as.Start(span, 0);
    as.m_gap = cpd.settings[UO_align_func_proto_gap].n;
+   as.m_star_style = (AlignStack::StarStyle)cpd.settings[UO_align_var_def_star_style].n;
+   as.m_amp_style  = (AlignStack::StarStyle)cpd.settings[UO_align_var_def_amp_style].n;
 
    as_br.Start(span, 0);
    as_br.m_gap = cpd.settings[UO_align_single_line_brace_gap].n;
@@ -1136,7 +1167,8 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
       if ((pc->type != CT_FUNC_CLASS) &&
           ((pc->flags & align_mask) == PCF_VAR_1ST) &&
           ((pc->level == (start->level + 1)) ||
-           (pc->level == 0)))
+           (pc->level == 0)) &&
+          pc->prev && (pc->prev->type != CT_MEMBER))
       {
          if (!did_this_line)
          {
@@ -1222,7 +1254,7 @@ chunk_t *align_nl_cont(chunk_t *start)
    }
 
    /* NL_CONT is always the last thing on a line */
-   while ((tmp = cs.Pop()) != NULL)
+   while ((tmp = cs.Pop_Back()) != NULL)
    {
       tmp->flags |= PCF_WAS_ALIGNED;
       tmp->column = max_col;
@@ -1268,7 +1300,7 @@ static CmtAlignType get_comment_align_type(chunk_t *cmt)
  * The series ends when more than align_right_cmt_span newlines are found.
  *
  * Interesting info:
- *  - least pysically allowed column
+ *  - least physically allowed column
  *  - intended column
  *  - least original cmt column
  *
@@ -1379,7 +1411,6 @@ static chunk_t *scan_ib_line(chunk_t *start, bool first_pass)
    chunk_t *prev_match = NULL;
    int     token_width;
    int     idx = 0;
-   bool    last_was_comment = false;
 
    /* Skip past C99 "[xx] =" stuff */
    if (start->type == CT_SQUARE_OPEN)
@@ -1428,7 +1459,6 @@ static chunk_t *scan_ib_line(chunk_t *start, bool first_pass)
             cpd.al[cpd.al_cnt].len  = token_width;
             cpd.al_cnt++;
             idx++;
-            last_was_comment = false;
          }
          else
          {
@@ -1467,14 +1497,8 @@ static chunk_t *scan_ib_line(chunk_t *start, bool first_pass)
          }
          prev_match = pc;
       }
-      last_was_comment = chunk_is_comment(pc);
       pc = chunk_get_next_nc(pc);
    }
-
-   //if (last_was_comment && (cpd.al[cpd.al_cnt - 1].type == CT_COMMA))
-   //{
-   //   cpd.al_cnt--;
-   //}
    return(pc);
 }
 
@@ -1687,8 +1711,6 @@ static void align_init_brace(chunk_t *start)
  * Aligns simple typedefs that are contained on a single line each.
  * This should be called after the typedef target is marked as a type.
  *
- * Won't align function typedefs.
- *
  * typedef int        foo_t;
  * typedef char       bar_t;
  * typedef const char cc_t;
@@ -1696,7 +1718,6 @@ static void align_init_brace(chunk_t *start)
 static void align_typedefs(int span)
 {
    chunk_t    *pc;
-   chunk_t    *c_type    = NULL;
    chunk_t    *c_typedef = NULL;
    AlignStack as;
 
@@ -1718,6 +1739,9 @@ static void align_typedefs(int span)
          if (pc->flags & PCF_ANCHOR)
          {
             as.Add(pc);
+            LOG_FMT(LALTD, "%s: typedef @ %d:%d, tag '%s' @ %d:%d\n",
+                    __func__, c_typedef->orig_line, c_typedef->orig_col,
+                    pc->text(), pc->orig_line, pc->orig_col);
             c_typedef = NULL;
          }
       }
@@ -1725,10 +1749,7 @@ static void align_typedefs(int span)
       {
          if (pc->type == CT_TYPEDEF)
          {
-            LOG_FMT(LALTD, "%s: line %d, col %d\n",
-                    __func__, pc->orig_line, pc->orig_col);
             c_typedef = pc;
-            c_type    = NULL;
          }
       }
 
@@ -1748,7 +1769,7 @@ static void align_left_shift(void)
    chunk_t    *start = NULL;
    AlignStack as;
 
-   as.Start(2);
+   as.Start(255);
 
    pc = chunk_get_head();
    while (pc != NULL)
@@ -1775,8 +1796,26 @@ static void align_left_shift(void)
       }
       else if (chunk_is_str(pc, "<<", 2))
       {
-         if (as.m_aligned.Empty())
+         if (pc->parent_type == CT_OPERATOR)
          {
+            /* Ignore operator<< */
+         }
+         else if (as.m_aligned.Empty())
+         {
+            /* check if the first one is actually on a blank line and then
+             * indent it. Eg:
+             *
+             *      cout
+             *          << "something";
+             */
+            chunk_t *prev = chunk_get_prev(pc);
+            if (prev && chunk_is_newline(prev))
+            {
+                indent_to_column(pc, pc->column_indent + cpd.settings[UO_indent_columns].n);
+                pc->column_indent = pc->column;
+                pc->flags |= PCF_DONT_INDENT;
+            }
+
             /* first one can be anywhere */
             as.Add(pc);
             start = pc;
@@ -1787,6 +1826,22 @@ static void align_left_shift(void)
             as.Add(pc);
          }
       }
+      else if (!as.m_aligned.Empty())
+      {
+          /* check if the given statement is on a line of its own, immediately following <<
+           * and then it. Eg:
+           *
+           *      cout <<
+           *          "something";
+           */
+          chunk_t *prev = chunk_get_prev(pc);
+          if (prev && chunk_is_newline(prev))
+          {
+              indent_to_column(pc, pc->column_indent + cpd.settings[UO_indent_columns].n);
+              pc->column_indent = pc->column;
+              pc->flags |= PCF_DONT_INDENT;
+          }
+      }
 
       pc = chunk_get_next(pc);
    }
@@ -1850,11 +1905,57 @@ static void align_oc_msg_colon(chunk_t *so)
               (tmp->type == CT_OC_MSG_NAME)))
          {
             nas.Add(tmp);
+            tmp->flags |= PCF_DONT_INDENT;
          }
          did_line = true;
       }
       pc = chunk_get_next(pc, CNAV_PREPROC);
    }
+
+   nas.m_skip_first = !cpd.settings[UO_align_oc_msg_colon_first].b;
+   cas.m_skip_first = !cpd.settings[UO_align_oc_msg_colon_first].b;
+
+   /* find the longest args that isn't the first one */
+   int     idx, len;
+   int     tlen, mlen = 0;
+   chunk_t *longest = NULL;
+
+   for (idx = 0, len = nas.m_aligned.Len(); idx < len; idx++)
+   {
+      tmp = nas.m_aligned.GetChunk(idx);
+
+      tlen = tmp->str.size();
+      if (tlen > mlen)
+      {
+         mlen = tlen;
+         if (idx != 0)
+         {
+            longest = tmp;
+         }
+      }
+   }
+
+   /* add spaces before the longest arg */
+   len = cpd.settings[UO_indent_oc_msg_colon].n;
+   if (longest && (len > 0))
+   {
+      chunk_t chunk;
+
+      chunk.type        = CT_SPACE;
+      chunk.orig_line   = longest->orig_line;
+      chunk.parent_type = CT_NONE;
+      chunk.level       = longest->level;
+      chunk.brace_level = longest->brace_level;
+      chunk.flags       = longest->flags & PCF_COPY_FLAGS;
+
+      /* start at one since we already indent for the '[' */
+      for (idx = 1; idx < len; idx++)
+      {
+         chunk.str.append(' ');
+      }
+
+      chunk_add_before(&chunk, longest);
+   }
    nas.End();
    cas.End();
 }
@@ -1939,7 +2040,8 @@ static void align_oc_decl_colon(void)
                 &&
                 ((tmp->type == CT_WORD) ||
                  (tmp->type == CT_TYPE) ||
-                 (tmp->type == CT_OC_MSG_DECL))
+                 (tmp->type == CT_OC_MSG_DECL) ||
+                 (tmp->type == CT_OC_MSG_SPEC))
                 &&
                 ((tmp2->type == CT_WORD) ||
                  (tmp2->type == CT_TYPE) ||
