@@ -168,7 +168,7 @@ void AlignStack::Add(chunk_t *start, int seqnum)
       while (((prev = chunk_get_prev(prev)) != NULL) &&
              (chunk_is_star(prev) ||
               chunk_is_addr(prev) ||
-              (chunk_is_str(prev, "(", 1) && (prev->parent_type == CT_TYPEDEF))))
+              (prev->type == CT_TPAREN_OPEN)))
       {
          /* do nothing - we want prev when this exits */
       }
@@ -189,7 +189,7 @@ void AlignStack::Add(chunk_t *start, int seqnum)
             ali  = prev;
             prev = chunk_get_prev(ali);
          }
-         if (chunk_is_str(prev, "(", 1) && (prev->parent_type == CT_TYPEDEF))
+         if (chunk_is_token(prev, CT_TPAREN_OPEN))
          {
             ali  = prev;
             prev = chunk_get_prev(ali);
@@ -228,7 +228,7 @@ void AlignStack::Add(chunk_t *start, int seqnum)
          gap = ali->column - (ref->column + ref->len());
       }
       tmp = ali;
-      if (chunk_is_str(tmp, "(", 1) && (tmp->parent_type == CT_TYPEDEF))
+      if (chunk_is_token(tmp, CT_TPAREN_OPEN))
       {
          tmp = chunk_get_next(tmp);
       }
@@ -257,7 +257,7 @@ void AlignStack::Add(chunk_t *start, int seqnum)
       ali->align.col_adj = col_adj;
       ali->align.ref     = ref;
       ali->align.start   = start;
-      m_aligned.Push(ali, seqnum);
+      m_aligned.Push_Back(ali, seqnum);
       m_last_added = 1;
 
       LOG_FMT(LAS, "Add-[%s]: line %d, col %d, adj %d : ref=[%s] endcol=%d\n",
@@ -295,7 +295,7 @@ void AlignStack::Add(chunk_t *start, int seqnum)
    else
    {
       /* The threshold check failed, so add it to the skipped list */
-      m_skipped.Push(start, seqnum);
+      m_skipped.Push_Back(start, seqnum);
       m_last_added = 2;
 
       LOG_FMT(LAS, "Add-skipped [%d/%d/%d]: line %d, col %d <= %d + %d\n",
@@ -356,7 +356,7 @@ void AlignStack::Flush()
          gap = pc->column - (pc->align.ref->column + pc->align.ref->len());
       }
       chunk_t *tmp = pc;
-      if (chunk_is_str(tmp, "(", 1) && (tmp->parent_type == CT_TYPEDEF))
+      if (tmp->type == CT_TPAREN_OPEN)
       {
          tmp = chunk_get_next(tmp);
       }
@@ -404,19 +404,30 @@ void AlignStack::Flush()
    {
       ce = m_aligned.Get(idx);
       pc = ce->m_pc;
+
+      tmp_col = m_max_col - pc->align.col_adj;
       if (idx == 0)
       {
+         if (m_skip_first && (pc->column != tmp_col))
+         {
+            LOG_FMT(LAS, "%s: %d:%d dropping first item due to skip_first\n", __func__,
+                    pc->orig_line, pc->orig_col);
+            m_skip_first = false;
+            m_aligned.Pop_Front();
+            Flush();
+            m_skip_first = true;
+            return;
+         }
          pc->flags |= PCF_ALIGN_START;
 
          pc->align.right_align = m_right_align;
          pc->align.amp_style   = (int)m_amp_style;
          pc->align.star_style  = (int)m_star_style;
-         pc->align.gap         = m_gap;
       }
+      pc->align.gap  = m_gap;
       pc->align.next = m_aligned.GetChunk(idx + 1);
 
       /* Indent the token, taking col_adj into account */
-      tmp_col = m_max_col - pc->align.col_adj;
       LOG_FMT(LAS, "%s: line %d: '%s' to col %d (adj=%d)\n", __func__,
               pc->orig_line, pc->str.c_str(), tmp_col, pc->align.col_adj);
       align_to_column(pc, tmp_col);
