@@ -70,7 +70,7 @@ static int preproc_start(struct parse_frame *frm, chunk_t *pc)
 }
 
 
-static void print_stack(int logsev, const char *str,
+static void print_stack(log_sev_t logsev, const char *str,
                         struct parse_frame *frm, chunk_t *pc)
 {
    if (log_sev_on(logsev))
@@ -105,7 +105,6 @@ static void print_stack(int logsev, const char *str,
 void brace_cleanup(void)
 {
    chunk_t            *pc;
-   chunk_t            vs_chunk;
    struct parse_frame frm;
    int                pp_level;
 
@@ -290,12 +289,15 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
    if (((frm->stmt_count == 0) || (frm->expr_count == 0)) &&
        !chunk_is_semicolon(pc) &&
        (pc->type != CT_BRACE_CLOSE) &&
-       (pc->type != CT_VBRACE_CLOSE))
+       (pc->type != CT_VBRACE_CLOSE) &&
+       !chunk_is_str(pc, ")", 1) &&
+       !chunk_is_str(pc, "]", 1))
    {
       pc->flags |= PCF_EXPR_START;
       pc->flags |= (frm->stmt_count == 0) ? PCF_STMT_START : 0;
-      LOG_FMT(LSTMT, "%d] 1.marked %s as stmt start st:%d ex:%d\n",
-              pc->orig_line, pc->str.c_str(), frm->stmt_count, frm->expr_count);
+      LOG_FMT(LSTMT, "%d] 1.marked %s as %s start st:%d ex:%d\n",
+              pc->orig_line, pc->str.c_str(), (pc->flags &PCF_STMT_START) ? "stmt" : "expr",
+              frm->stmt_count, frm->expr_count);
    }
    frm->stmt_count++;
    frm->expr_count++;
@@ -577,12 +579,14 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
        (pc->type == CT_BRACE_CLOSE) ||
        (pc->type == CT_VBRACE_CLOSE) ||
        ((pc->type == CT_SPAREN_OPEN) && (pc->parent_type == CT_FOR)) ||
+       (pc->type == CT_COLON) ||
+       (pc->type == CT_OC_END) ||
        (chunk_is_semicolon(pc) &&
         (frm->pse[frm->pse_tos].type != CT_PAREN_OPEN) &&
         (frm->pse[frm->pse_tos].type != CT_FPAREN_OPEN) &&
         (frm->pse[frm->pse_tos].type != CT_SPAREN_OPEN)))
    {
-      LOG_FMT(LSTMT, "%s: %d> reset stmt on %s\n",
+      LOG_FMT(LSTMT, "%s: %d> reset1 stmt on %s\n",
               __func__, pc->orig_line, pc->str.c_str());
       frm->stmt_count = 0;
       frm->expr_count = 0;
@@ -598,9 +602,11 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
        (pc->type == CT_BOOL) ||
        (pc->type == CT_MINUS) ||
        (pc->type == CT_PLUS) ||
+       (pc->type == CT_CARET) ||
        (pc->type == CT_ANGLE_OPEN) ||
        (pc->type == CT_ANGLE_CLOSE) ||
        (pc->type == CT_RETURN) ||
+       (pc->type == CT_THROW) ||
        (pc->type == CT_GOTO) ||
        (pc->type == CT_CONTINUE) ||
        (pc->type == CT_PAREN_OPEN) ||
@@ -958,7 +964,7 @@ bool close_statement(struct parse_frame *frm, chunk_t *pc)
    {
       frm->stmt_count = 0;
       frm->expr_count = 0;
-      LOG_FMT(LSTMT, "%s: %d> reset stmt on %s\n",
+      LOG_FMT(LSTMT, "%s: %d> reset2 stmt on %s\n",
               __func__, pc->orig_line, pc->str.c_str());
    }
 
