@@ -29,6 +29,11 @@ static bool should_add_braces(chunk_t *vbopen);
 
 void do_braces(void)
 {
+   if (cpd.settings[UO_mod_full_brace_if_chain].b)
+   {
+      mod_full_brace_if_chain();
+   }
+
    if (((cpd.settings[UO_mod_full_brace_if].a |
          cpd.settings[UO_mod_full_brace_do].a |
          cpd.settings[UO_mod_full_brace_for].a |
@@ -49,11 +54,6 @@ void do_braces(void)
       convert_vbrace_to_brace();
    }
 
-   if (cpd.settings[UO_mod_full_brace_if_chain].b)
-   {
-      mod_full_brace_if_chain();
-   }
-
    /* Mark one-liners */
    chunk_t *pc;
    chunk_t *br_open;
@@ -211,9 +211,15 @@ static bool can_remove_braces(chunk_t *bopen)
 
    LOG_FMT(LBRDEL, "%s: start on %d : ", __func__, bopen->orig_line);
 
-   pc = chunk_get_next_nc(bopen, CNAV_PREPROC);
+   pc = chunk_get_next_nc(bopen, CNAV_ALL);
    while ((pc != NULL) && (pc->level >= level))
    {
+      if (pc->flags & PCF_IN_PREPROC)
+      {
+         /* Cannot remove braces that contain a preprocessor */
+         return(false);
+      }
+
       if (chunk_is_newline(pc))
       {
          nl_count += pc->nl_count;
@@ -281,7 +287,7 @@ static bool can_remove_braces(chunk_t *bopen)
          }
       }
       prev = pc;
-      pc   = chunk_get_next_nc(pc, CNAV_PREPROC);
+      pc   = chunk_get_next_nc(pc);
    }
 
    if (pc == NULL)
@@ -494,7 +500,7 @@ static void convert_brace(chunk_t *br)
 {
    chunk_t *tmp;
 
-   if (br == NULL)
+   if (!br || (br->flags & PCF_KEEP_BRACE))
    {
       return;
    }
@@ -600,7 +606,8 @@ static void convert_vbrace_to_brace(void)
       if ((((pc->parent_type == CT_IF) ||
             (pc->parent_type == CT_ELSE) ||
             (pc->parent_type == CT_ELSEIF)) &&
-           ((cpd.settings[UO_mod_full_brace_if].a & AV_ADD) != 0))
+           ((cpd.settings[UO_mod_full_brace_if].a & AV_ADD) != 0) &&
+            !cpd.settings[UO_mod_full_brace_if_chain].b)
           ||
           ((pc->parent_type == CT_FOR) &&
            ((cpd.settings[UO_mod_full_brace_for].a & AV_ADD) != 0))
@@ -1037,6 +1044,7 @@ static void process_if_chain(chunk_t *br_start)
       LOG_FMT(LBRCH, "%s: add braces on lines[%d]:", __func__, br_cnt);
       while (--br_cnt >= 0)
       {
+         braces[br_cnt]->flags |= PCF_KEEP_BRACE;
          if ((braces[br_cnt]->type == CT_VBRACE_OPEN) ||
              (braces[br_cnt]->type == CT_VBRACE_CLOSE))
          {
