@@ -291,14 +291,16 @@ static void indent_pse_push(struct parse_frame& frm, chunk_t *pc)
       LOG_FMT(LINDPSE, "%4d] (pp=%d) OPEN  [%d,%s] level=%d\n",
               pc->orig_line, cpd.pp_level, frm.pse_tos, get_token_name(pc->type), pc->level);
 
-      frm.pse[frm.pse_tos].pc         = pc;
-      frm.pse[frm.pse_tos].type       = pc->type;
-      frm.pse[frm.pse_tos].level      = pc->level;
-      frm.pse[frm.pse_tos].open_line  = pc->orig_line;
-      frm.pse[frm.pse_tos].ref        = ++ref;
-      frm.pse[frm.pse_tos].in_preproc = (pc->flags & PCF_IN_PREPROC) != 0;
-      frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab;
-      frm.pse[frm.pse_tos].non_vardef = false;
+      frm.pse[frm.pse_tos].pc          = pc;
+      frm.pse[frm.pse_tos].type        = pc->type;
+      frm.pse[frm.pse_tos].level       = pc->level;
+      frm.pse[frm.pse_tos].open_line   = pc->orig_line;
+      frm.pse[frm.pse_tos].ref         = ++ref;
+      frm.pse[frm.pse_tos].in_preproc  = (pc->flags & PCF_IN_PREPROC) != 0;
+      frm.pse[frm.pse_tos].indent_tab  = frm.pse[frm.pse_tos - 1].indent_tab;
+      frm.pse[frm.pse_tos].indent_cont = frm.pse[frm.pse_tos - 1].indent_cont;
+      frm.pse[frm.pse_tos].non_vardef  = false;
+      memcpy(&frm.pse[frm.pse_tos].ip, &frm.pse[frm.pse_tos - 1].ip, sizeof(frm.pse[frm.pse_tos].ip));
    }
 }
 
@@ -375,14 +377,45 @@ static int token_indent(c_token_t type)
    } while (0)
 
 
-/**
- * Determines the continuation indent if indent_continue is non-zero.
- */
-static int continue_indent(chunk_t *pc)
+static int calc_indent_continue(struct parse_frame& frm, int pse_tos)
 {
-   return(1 + (pc->level * cpd.settings[UO_indent_columns].n) +
-          cpd.settings[UO_indent_continue].n);
+   int ic = cpd.settings[UO_indent_continue].n;
 
+   if ((ic < 0) && frm.pse[pse_tos].indent_cont)
+   {
+      return frm.pse[pse_tos].indent;
+   }
+   return frm.pse[pse_tos].indent + abs(ic);
+}
+
+
+/**
+ * We are on a '{' that has parent = OC_BLOCK_EXPR
+ * find the column of the param tag
+ */
+static chunk_t *oc_msg_block_indent(chunk_t *pc)
+{
+   chunk_t *tmp = chunk_get_prev_nc(pc);
+
+   if (chunk_is_paren_close(tmp))
+   {
+      tmp = chunk_get_prev_nc(chunk_skip_to_match_rev(tmp));
+   }
+   if (!tmp || (tmp->type != CT_OC_BLOCK_CARET))
+   {
+      return NULL;
+   }
+   tmp = chunk_get_prev_nc(tmp);
+   if (!tmp || (tmp->type != CT_OC_COLON))
+   {
+      return NULL;
+   }
+   tmp = chunk_get_prev_nc(tmp);
+   if (!tmp || ((tmp->type != CT_OC_MSG_NAME) && (tmp->type != CT_OC_MSG_FUNC)))
+   {
+      return NULL;
+   }
+   return tmp;
 }
 
 
@@ -402,13 +435,15 @@ void indent_text(void)
    int                indent_size = cpd.settings[UO_indent_columns].n;
    int                tmp;
    struct parse_frame frm;
-   bool               in_preproc = false, was_preproc = false;
+   bool               in_preproc = false;
    int                indent_column;
    int                parent_token_indent = 0;
    int                xml_indent          = 0;
    bool               token_used;
    int                sql_col      = 0;
    int                sql_orig_col = 0;
+   bool               in_func_def = false;
+   c_token_t          memtype;
 
    memset(&frm, 0, sizeof(frm));
    cpd.frame_count = 0;
@@ -422,15 +457,44 @@ void indent_text(void)
    pc = chunk_get_head();
    while (pc != NULL)
    {
-      /* Handle proprocessor transitions */
-      was_preproc = in_preproc;
-      in_preproc  = (pc->flags & PCF_IN_PREPROC) != 0;
+      /* Handle preprocessor transitions */
+      in_preproc = (pc->flags & PCF_IN_PREPROC) != 0;
 
       if (cpd.settings[UO_indent_brace_parent].b)
       {
          parent_token_indent = token_indent(pc->parent_type);
       }
 
+      /* Handle "force indentation of function definition to start in column 1" */
+      if (cpd.settings[UO_indent_func_def_force_col1].b)
+      {
+         if (!in_func_def)
+         {
+            next = chunk_get_next_ncnl(pc);
+            if ((pc->parent_type == CT_FUNC_DEF) ||
+                ((pc->type == CT_COMMENT) &&
+                 (next != NULL) &&
+                 (next->parent_type == CT_FUNC_DEF)))
+            {
+               in_func_def = true;
+               indent_pse_push(frm, pc);
+               frm.pse[frm.pse_tos].indent_tmp = 1;
+               frm.pse[frm.pse_tos].indent     = 1;
+               frm.pse[frm.pse_tos].indent_tab = 1;
+            }
+         }
+         else
+         {
+            prev = chunk_get_prev(pc);
+            if ((prev->type == CT_BRACE_CLOSE) &&
+                (prev->parent_type == CT_FUNC_DEF))
+            {
+               in_func_def = false;
+               indent_pse_pop(frm, pc);
+            }
+         }
+      }
+
       /* Clean up after a #define, etc */
       if (!in_preproc)
       {
@@ -485,9 +549,10 @@ void indent_text(void)
          {
             next = chunk_get_next(pc);
             /* Hack to get the logs to look right */
+            memtype = next->type;
             next->type = CT_PP_IF_INDENT;
             indent_pse_push(frm, next);
-            next->type = CT_PP_IF;
+            next->type = memtype;
 
             /* Indent one level */
             frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + indent_size;
@@ -504,7 +569,8 @@ void indent_text(void)
          {
             frm.pse[frm.pse_tos].indent_tmp = cpd.settings[UO_pp_define_at_level].b ?
                                               frm.pse[frm.pse_tos - 1].indent_tmp : 1;
-            frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos].indent_tmp + indent_size;
+            frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos].indent_tmp + indent_size;
+            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
          }
          else
          {
@@ -607,7 +673,8 @@ void indent_text(void)
                  (pc->type == CT_COMMA) ||
                  (pc->type == CT_BRACE_OPEN) ||
                  (pc->type == CT_SPAREN_CLOSE) ||
-                 ((pc->type == CT_SQUARE_OPEN) && (pc->parent_type == CT_ASSIGN))))
+                 ((pc->type == CT_SQUARE_OPEN) && (pc->parent_type == CT_ASSIGN))) &&
+                 (pc->parent_type != CT_CPP_LAMBDA))
             {
                indent_pse_pop(frm, pc);
             }
@@ -673,6 +740,16 @@ void indent_text(void)
                indent_pse_pop(frm, pc);
             }
 
+            /* a typedef and an OC SCOPE ('-' or '+') ends with a semicolon or
+             * brace open */
+            if ((frm.pse[frm.pse_tos].type == CT_TYPEDEF) &&
+                (chunk_is_semicolon(pc) ||
+                 chunk_is_paren_open(pc) ||
+                 (pc->type == CT_BRACE_OPEN)))
+            {
+               indent_pse_pop(frm, pc);
+            }
+
             /* an SQL EXEC is ended with a semicolon */
             if ((frm.pse[frm.pse_tos].type == CT_SQL_EXEC) &&
                 chunk_is_semicolon(pc))
@@ -735,7 +812,6 @@ void indent_text(void)
                           (pc->parent_type != CT_CLASS)) &&
                          (!cpd.settings[UO_indent_braces_no_struct].b ||
                           (pc->parent_type != CT_STRUCT)));
-
       }
 
       if (pc->type == CT_BRACE_CLOSE)
@@ -745,6 +821,12 @@ void indent_text(void)
             /* Indent the brace to match the open brace */
             indent_column_set(frm.pse[frm.pse_tos].brace_indent);
 
+            if (frm.pse[frm.pse_tos].ip.ref)
+            {
+               pc->indent.ref   = frm.pse[frm.pse_tos].ip.ref;
+               pc->indent.delta = 0;
+            }
+
             indent_pse_pop(frm, pc);
             frm.level--;
          }
@@ -768,8 +850,30 @@ void indent_text(void)
 
          if (frm.paren_count != 0)
          {
-            /* We are inside ({ ... }) -- indent one tab from the paren */
-            frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
+            if (frm.pse[frm.pse_tos].pc->parent_type == CT_OC_BLOCK_EXPR)
+            {
+               if ((pc->flags & PCF_IN_OC_MSG) &&
+                   cpd.settings[UO_indent_oc_block_msg].n)
+               {
+                  frm.pse[frm.pse_tos].ip.ref   = oc_msg_block_indent(pc);
+                  frm.pse[frm.pse_tos].ip.delta = cpd.settings[UO_indent_oc_block_msg].n;
+               }
+
+               if (cpd.settings[UO_indent_oc_block].b)
+               {
+                  frm.pse[frm.pse_tos].indent = 1 + (pc->level * indent_size);
+                  indent_column_set(frm.pse[frm.pse_tos].indent - indent_size);
+               }
+               else
+               {
+                  frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
+               }
+            }
+            else
+            {
+               /* We are inside ({ ... }) -- indent one tab from the paren */
+               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent_tmp + indent_size;
+            }
          }
          else
          {
@@ -1010,7 +1114,7 @@ void indent_text(void)
             prev = chunk_get_prev(pc);
             if (chunk_is_newline(prev))
             {
-               frm.pse[frm.pse_tos].indent += 2;
+               frm.pse[frm.pse_tos].indent += cpd.settings[UO_indent_ctor_init_leading].n;
 
                if (cpd.settings[UO_indent_ctor_init].n > 0)
                {
@@ -1039,6 +1143,8 @@ void indent_text(void)
          /* Open parens and squares - never update indent_column, unless right
           * after a newline.
           */
+         bool skipped = false;
+
          indent_pse_push(frm, pc);
          frm.pse[frm.pse_tos].indent = pc->column + pc->len();
 
@@ -1077,6 +1183,7 @@ void indent_text(void)
                    (frm.pse[idx].type != CT_ASSIGN_NL))
             {
                idx--;
+               skipped = true;
             }
             frm.pse[frm.pse_tos].indent = frm.pse[idx].indent + indent_size;
             if (cpd.settings[UO_indent_func_param_double].b)
@@ -1094,11 +1201,13 @@ void indent_text(void)
             if (chunk_is_newline(next))
             {
                int sub = 1;
-               if (frm.pse[frm.pse_tos - 1].type == CT_ASSIGN)
+               if (frm.pse[frm.pse_tos - 1].type == CT_ASSIGN ||
+                   frm.pse[frm.pse_tos - 1].type == CT_RETURN)
                {
                   sub = 2;
                }
                frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - sub].indent + indent_size;
+               skipped = true;
             }
             else
             {
@@ -1120,11 +1229,17 @@ void indent_text(void)
             frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent + indent_size;
             indent_column_set(frm.pse[frm.pse_tos].indent);
          }
-         if (cpd.settings[UO_indent_continue].n != 0)
+         if ((cpd.settings[UO_indent_continue].n != 0) && (!skipped))
          {
-            frm.pse[frm.pse_tos].indent = continue_indent(pc);
-            /* the continuation indent applies to the next line, not the current */
-            //indent_column_set(frm.pse[frm.pse_tos].indent);
+            frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent;
+            if ((pc->level == pc->brace_level) &&
+                (((pc->type == CT_FPAREN_OPEN) && (pc->parent_type != CT_FUNC_CTOR_VAR)) ||
+                 (pc->type == CT_SPAREN_OPEN)))
+            {
+               //frm.pse[frm.pse_tos].indent += abs(cpd.settings[UO_indent_continue].n);
+               frm.pse[frm.pse_tos].indent = calc_indent_continue(frm, frm.pse_tos);
+               frm.pse[frm.pse_tos].indent_cont = true;
+            }
          }
          frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
          frm.paren_count++;
@@ -1151,9 +1266,18 @@ void indent_text(void)
          if (next != NULL)
          {
             indent_pse_push(frm, pc);
-            if (cpd.settings[UO_indent_continue].n)
+            if (cpd.settings[UO_indent_continue].n != 0)
             {
-               frm.pse[frm.pse_tos].indent = continue_indent(pc);
+               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent;
+               if ((pc->level == pc->brace_level) &&
+                   ((pc->type != CT_ASSIGN) ||
+                    ((pc->parent_type != CT_FUNC_PROTO) && (pc->parent_type != CT_FUNC_DEF))))
+               {
+                  //frm.pse[frm.pse_tos].indent += abs(cpd.settings[UO_indent_continue].n);
+                  frm.pse[frm.pse_tos].indent = calc_indent_continue(frm, frm.pse_tos);
+                  frm.pse[frm.pse_tos].indent_cont = true;
+
+               }
             }
             else if (chunk_is_newline(next) || !cpd.settings[UO_indent_align_assign].b)
             {
@@ -1188,16 +1312,19 @@ void indent_text(void)
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos - 1].indent;
          }
       }
-      else if (pc->type == CT_OC_SCOPE)
+      else if ((pc->type == CT_OC_SCOPE) || (pc->type == CT_TYPEDEF))
       {
          indent_pse_push(frm, pc);
          if (cpd.settings[UO_indent_continue].n != 0)
          {
-            frm.pse[frm.pse_tos].indent += cpd.settings[UO_indent_continue].n;
+            //frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent +
+            //                              abs(cpd.settings[UO_indent_continue].n);
+            frm.pse[frm.pse_tos].indent = calc_indent_continue(frm, frm.pse_tos - 1);
+            frm.pse[frm.pse_tos].indent_cont = true;
          }
          else
          {
-            frm.pse[frm.pse_tos].indent += indent_size;
+            frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent + indent_size;
          }
       }
       else
@@ -1209,14 +1336,16 @@ void indent_text(void)
        * Handle variable definition continuation indenting
        */
       if ((vardefcol == 0) &&
-          (pc->type == CT_WORD) &&
+          ((pc->type == CT_WORD) || (pc->type == CT_FUNC_CTOR_VAR)) &&
           ((pc->flags & PCF_IN_FCN_DEF) == 0) &&
           ((pc->flags & PCF_VAR_1ST_DEF) == PCF_VAR_1ST_DEF))
       {
          if (cpd.settings[UO_indent_continue].n != 0)
          {
-            vardefcol = frm.pse[frm.pse_tos].indent +
-                        cpd.settings[UO_indent_continue].n;
+            //vardefcol = frm.pse[frm.pse_tos].indent +
+            //            abs(cpd.settings[UO_indent_continue].n);
+            vardefcol = calc_indent_continue(frm, frm.pse_tos);
+            frm.pse[frm.pse_tos].indent_cont = true;
          }
          else if (cpd.settings[UO_indent_var_def_cont].b ||
                   chunk_is_newline(chunk_get_prev(pc)))
@@ -1241,8 +1370,14 @@ void indent_text(void)
       {
          pc->column_indent = frm.pse[frm.pse_tos].indent_tab;
 
-         LOG_FMT(LINDENT2, "%s: %d] %d for %s\n",
-                 __func__, pc->orig_line, pc->column_indent, pc->str.c_str());
+         if (frm.pse[frm.pse_tos].ip.ref)
+         {
+            pc->indent.ref   = frm.pse[frm.pse_tos].ip.ref;
+            pc->indent.delta = frm.pse[frm.pse_tos].ip.delta;
+         }
+
+         LOG_FMT(LINDENT2, "%s: %d] %d/%d for %s\n",
+                 __func__, pc->orig_line, pc->column_indent, indent_column, pc->str.c_str());
 
          /**
           * Check for special continuations.
@@ -1277,7 +1412,7 @@ void indent_text(void)
          }
          else if ((vardefcol > 0) &&
                   (pc->level == pc->brace_level) &&
-                  (pc->type == CT_WORD) &&
+                  ((pc->type == CT_WORD) || (pc->type == CT_FUNC_CTOR_VAR)) &&
                   (prev != NULL) &&
                   ((prev->type == CT_COMMA) ||
                    (prev->type == CT_TYPE) ||
@@ -1288,7 +1423,7 @@ void indent_text(void)
                     __func__, pc->orig_line, vardefcol);
             reindent_line(pc, vardefcol);
          }
-         else if ((pc->type == CT_STRING) && (prev->type == CT_STRING) &&
+         else if ((pc->type == CT_STRING) && (prev != NULL) && (prev->type == CT_STRING) &&
                   cpd.settings[UO_indent_align_string].b)
          {
             tmp = (xml_indent != 0) ? xml_indent : prev->column;
@@ -1505,6 +1640,7 @@ void indent_text(void)
    }
 
    quick_align_again();
+   quick_indent_again();
 }
 
 
@@ -1771,7 +1907,7 @@ void indent_preproc(void)
 
       /* Mark as already handled if not region stuff or in column 1 */
       if ((!cpd.settings[UO_pp_indent_at_level].b ||
-           (pc->brace_level <= (pc->parent_type == CT_PP_DEFINE ? 1 : 0))) &&
+           (pc->brace_level <= ((pc->parent_type == CT_PP_DEFINE) ? 1 : 0))) &&
           (pc->parent_type != CT_PP_REGION) &&
           (pc->parent_type != CT_PP_ENDREGION))
       {
