@@ -39,16 +39,22 @@ static const chunk_tag_t keywords[] =
    { "@protocol",        CT_OC_PROTOCOL,  LANG_OC | LANG_CPP | LANG_C                                                 },
    { "@selector",        CT_OC_SEL,       LANG_OC | LANG_CPP | LANG_C                                                 },
    { "@synthesize",      CT_OC_DYNAMIC,   LANG_OC | LANG_CPP | LANG_C                                                 },
+   { "@throw",           CT_THROW,        LANG_OC                                                                     },
    { "@try",             CT_TRY,          LANG_OC | LANG_CPP | LANG_C                                                 },
    { "_Bool",            CT_TYPE,         LANG_CPP                                                                    },
    { "_Complex",         CT_TYPE,         LANG_CPP                                                                    },
    { "_Imaginary",       CT_TYPE,         LANG_CPP                                                                    },
    { "__attribute__",    CT_ATTRIBUTE,    LANG_C | LANG_CPP                                                           },
+   { "__block",          CT_QUALIFIER,    LANG_OC                                                                     },
    { "__const__",        CT_QUALIFIER,    LANG_C | LANG_CPP                                                           },
+   { "__except",         CT_CATCH,        LANG_C | LANG_CPP                                                           },
+   { "__finally",        CT_FINALLY,      LANG_C | LANG_CPP                                                           },
    { "__inline__",       CT_QUALIFIER,    LANG_C | LANG_CPP                                                           },
    { "__restrict",       CT_QUALIFIER,    LANG_C | LANG_CPP                                                           },
    { "__signed__",       CT_TYPE,         LANG_C | LANG_CPP                                                           },
+   { "__thread",         CT_QUALIFIER,    LANG_C | LANG_CPP                                                           },
    { "__traits",         CT_QUALIFIER,    LANG_D                                                                      },
+   { "__try",            CT_TRY,          LANG_C | LANG_CPP                                                           },
    { "__typeof__",       CT_SIZEOF,       LANG_C | LANG_CPP                                                           },
    { "__volatile__",     CT_QUALIFIER,    LANG_C | LANG_CPP                                                           },
    { "abstract",         CT_QUALIFIER,    LANG_CS | LANG_D | LANG_JAVA | LANG_VALA | LANG_ECMA                        },
@@ -151,7 +157,7 @@ static const chunk_tag_t keywords[] =
    { "inout",            CT_QUALIFIER,    LANG_D                                                                      },
    { "instanceof",       CT_SIZEOF,       LANG_JAVA | LANG_ECMA                                                       },
    { "int",              CT_TYPE,         LANG_ALLC                                                                   },
-   { "interface",        CT_CLASS,        LANG_CS | LANG_D | LANG_JAVA | LANG_VALA | LANG_ECMA                        },
+   { "interface",        CT_CLASS,        LANG_C | LANG_CPP | LANG_CS | LANG_D | LANG_JAVA | LANG_VALA | LANG_ECMA    },
    { "internal",         CT_QUALIFIER,    LANG_CS                                                                     },
    { "invariant",        CT_INVARIANT,    LANG_D                                                                      },
    { "ireal",            CT_TYPE,         LANG_D                                                                      },
@@ -177,7 +183,8 @@ static const chunk_tag_t keywords[] =
    { "or_eq",            CT_SASSIGN,      LANG_C | LANG_CPP                                                           },
    { "out",              CT_QUALIFIER,    LANG_CS | LANG_D | LANG_VALA                                                },
    { "override",         CT_QUALIFIER,    LANG_CS | LANG_D | LANG_VALA                                                },
-   { "package",          CT_NAMESPACE,    LANG_D | LANG_JAVA | LANG_ECMA                                              },
+   { "package",          CT_PRIVATE,      LANG_D                                                                      },
+   { "package",          CT_PACKAGE,      LANG_ECMA | LANG_JAVA                                                       },
    { "params",           CT_TYPE,         LANG_CS                                                                     },
    { "pragma",           CT_PP_PRAGMA,    LANG_ALL | FLAG_PP                                                          }, // PAWN
    { "private",          CT_PRIVATE,      LANG_ALLC                                                                   }, // not C
@@ -202,7 +209,7 @@ static const chunk_tag_t keywords[] =
    { "signals",          CT_PRIVATE,      LANG_CPP                                                                    },
    { "signed",           CT_TYPE,         LANG_C | LANG_CPP                                                           },
    { "sizeof",           CT_SIZEOF,       LANG_C | LANG_CPP | LANG_CS | LANG_PAWN                                     }, // PAWN
-   { "sleep",            CT_FUNCTION,     LANG_C | LANG_CPP | LANG_CS | LANG_PAWN                                     }, // PAWN
+   { "sleep",            CT_SIZEOF,       LANG_PAWN                                                                   }, // PAWN
    { "stackalloc",       CT_NEW,          LANG_CS                                                                     },
    { "state",            CT_STATE,        LANG_PAWN                                                                   }, // PAWN
    { "static",           CT_QUALIFIER,    LANG_ALL                                                                    }, // PAWN
@@ -223,7 +230,7 @@ static const chunk_tag_t keywords[] =
    { "true",             CT_WORD,         LANG_CPP | LANG_CS | LANG_D | LANG_JAVA | LANG_VALA                         },
    { "try",              CT_TRY,          LANG_CPP | LANG_CS | LANG_D | LANG_JAVA | LANG_ECMA                         },
    { "tryinclude",       CT_PP_INCLUDE,   LANG_PAWN | FLAG_PP                                                         }, // PAWN
-   { "typedef",          CT_TYPEDEF,      LANG_C | LANG_CPP | LANG_D                                                  },
+   { "typedef",          CT_TYPEDEF,      LANG_C | LANG_CPP | LANG_D | LANG_OC                                        },
    { "typeid",           CT_SIZEOF,       LANG_C | LANG_CPP | LANG_D                                                  },
    { "typename",         CT_TYPENAME,     LANG_CPP                                                                    },
    { "typeof",           CT_SIZEOF,       LANG_C | LANG_CPP | LANG_CS | LANG_D | LANG_VALA | LANG_ECMA                },
@@ -453,19 +460,6 @@ int load_keyword_file(const char *filename)
 }
 
 
-void output_types(FILE *pfile)
-{
-   if (dkwm.size() > 0)
-   {
-      fprintf(pfile, "-== User Types ==-\n");
-      for (dkwmap::iterator it = dkwm.begin(); it != dkwm.end(); ++it)
-      {
-         fprintf(pfile, "%s\n", (*it).first.c_str());
-      }
-   }
-}
-
-
 void print_keywords(FILE *pfile)
 {
    for (dkwmap::iterator it = dkwm.begin(); it != dkwm.end(); ++it)
