@@ -10,7 +10,7 @@
 #ifndef LOG_LEVELS_H_INCLUDED
 #define LOG_LEVELS_H_INCLUDED
 
-enum
+enum log_sev_t
 {
    LSYS      = 0,
    LERR      = 1,
@@ -94,6 +94,12 @@ enum
    LBLANK    = 80,    /* Blank Lines */
    LOBJCWORD = 81,    /* Convert keyword to CT_WORD in certain circumstances */
    LCHANGE   = 82,    /* something changed */
+   LCONTTEXT = 83,    /* comment cont_text set */
+   LANNOT    = 84,    /* Java annotation */
+   LOCBLK    = 85,    /* OC Block stuff */
+   LFLPAREN  = 86,    /* Flag paren */
+   LOCMSGD   = 87,    /* OC Message declaration */
+   LINDENTAG = 88,    /* indent again */
 };
 
 #endif   /* LOG_LEVELS_H_INCLUDED */
