@@ -14,12 +14,13 @@
 #include <cstdio>
 #include <stdarg.h>
 #include "unc_ctype.h"
+#include "log_levels.h"
 
 
 /** Private log structure */
 struct log_buf
 {
-   log_buf() : log_file(0), sev(0), in_log(0), buf_len(0), show_hdr(false)
+   log_buf() : log_file(0), sev(LSYS), in_log(0), buf_len(0), show_hdr(false)
    {
    }
 
@@ -44,9 +45,9 @@ void log_init(FILE *log_file)
 {
    /* set the top 3 severities */
    logmask_set_all(g_log.mask, false);
-   log_set_sev(0, true);
-   log_set_sev(1, true);
-   log_set_sev(2, true);
+   log_set_sev(LSYS, true);
+   log_set_sev(LERR, true);
+   log_set_sev(LWARN, true);
 
    g_log.log_file = (log_file != NULL) ? log_file : stderr;
 }
@@ -116,8 +117,6 @@ void log_get_mask(log_mask_t& mask)
  */
 static void log_flush(bool force_nl)
 {
-   int dummy;  /* get the compiler to shut up */
-
    if (g_log.buf_len > 0)
    {
       if (force_nl && (g_log.buf[g_log.buf_len - 1] != '\n'))
@@ -125,7 +124,10 @@ static void log_flush(bool force_nl)
          g_log.buf[g_log.buf_len++] = '\n';
          g_log.buf[g_log.buf_len]   = 0;
       }
-      dummy = fwrite(g_log.buf, 1, g_log.buf_len, g_log.log_file);
+      if (fwrite(g_log.buf, g_log.buf_len, 1,g_log .log_file) != 1)
+      {
+         /* maybe we should log something to complain... =) */
+      }
 
       g_log.buf_len = 0;
    }
