@@ -53,6 +53,7 @@ enum tokenpos_e
    TP_TRAIL       = 8,     /* at the end of a line or trailing if wrapped line */
    TP_TRAIL_BREAK = (TP_TRAIL | TP_BREAK),
    TP_TRAIL_FORCE = (TP_TRAIL | TP_FORCE),
+   TP_JOIN        = 16,    /* remove newlines on both sides */
 };
 
 union op_val_t
@@ -144,6 +145,7 @@ enum uncrustify_options
 
    UO_indent_col1_comment,       // indent comments in column 1
 
+   UO_indent_func_def_force_col1,// force indentation of function definition to start in column 1
    UO_indent_func_call_param,    // indent continued function calls to indent_columns
    UO_indent_func_proto_param,   // same, but for function protos
    UO_indent_func_def_param,     // same, but for function defs
@@ -162,6 +164,7 @@ enum uncrustify_options
    UO_indent_class,                         // indent stuff inside class braces
    UO_indent_class_colon,                   // indent stuff after a class colon
 
+   UO_indent_ctor_init_leading,             // virtual indent from the ':' for member initializers. Default is 2. (applies to the leading colon case)
    UO_indent_ctor_init,                     // additional indenting for ctor initializer lists
 
    UO_indent_member,                        // indent lines broken at a member '.' or '->'
@@ -170,6 +173,9 @@ enum uncrustify_options
    UO_indent_relative_single_line_comments, // indent single line ('//') comments after code
    UO_indent_preserve_sql,                  // preserve indent of EXEC SQL statement body
    UO_indent_align_assign,
+   UO_indent_oc_block,
+   UO_indent_oc_block_msg,
+   UO_indent_oc_msg_colon,
 
    UO_indent_else_if,
    UO_indent_var_def_blk,        // indent a variable def block that appears at the top
@@ -192,13 +198,17 @@ enum uncrustify_options
    UO_sp_after_byref,           // space after a '&'  as in 'int& var'
 
    UO_sp_after_type,            // space between type and word
+   UO_sp_before_template_paren, // D: "template Foo("
 
    UO_sp_inside_fparen,         // space inside 'foo( xxx )' vs 'foo(xxx)'
    UO_sp_inside_fparens,        // space inside 'foo( )' vs 'foo()'
+   UO_sp_inside_tparen,
+   UO_sp_after_tparen_close,
    UO_sp_inside_paren,          // space inside '+ ( xxx )' vs '+ (xxx)'
    UO_sp_inside_square,         // space inside 'byte[ 5 ]' vs 'byte[5]'
    UO_sp_inside_sparen,         // space inside 'if( xxx )' vs 'if(xxx)'
    UO_sp_inside_sparen_close,
+   UO_sp_inside_sparen_open,
    UO_sp_inside_angle,          // space inside '<>', as in '<class T>'
 
    UO_sp_before_sparen,         // space before '(' of 'if/for/while/switch'
@@ -213,6 +223,7 @@ enum uncrustify_options
    UO_sp_angle_paren,           // space between '<>' and '(' in "a = new List<byte>();"
    UO_sp_angle_word,            // space between '<>' and a word in "List<byte> a;"
    UO_sp_angle_shift,           // '> >' vs '>>'
+   UO_sp_permit_cpp11_shift,    // '>>' vs '> >' for C++11 code
 
    UO_sp_before_square,         // space before single '['
    UO_sp_before_squares,        // space before '[]', as in 'byte []'
@@ -232,9 +243,12 @@ enum uncrustify_options
    UO_sp_arith,                 // space around + - / * etc
    UO_sp_bool,                  // space around || &&
    UO_sp_pp_concat,             // space around ##
-   UO_sp_pp_stringify,          // space around #
+   UO_sp_pp_stringify,          // space after #
+   UO_sp_before_pp_stringify,   // space before # in a #define x(y) L#y
    UO_sp_compare,               // space around < > ==, etc
    UO_sp_assign,                // space around =, +=, etc
+   UO_sp_cpp_lambda_assign,     // space around the capture spec [=](...){...}
+   UO_sp_cpp_lambda_paren,      // space after the capture spec [] (...){...}
    UO_sp_assign_default,        // space around '=' in prototype
    UO_sp_before_assign,         // space before =, +=, etc
    UO_sp_after_assign,          // space after =, +=, etc
@@ -255,6 +269,7 @@ enum uncrustify_options
    UO_sp_attribute_paren,       // space between '__attribute__' and '('
    UO_sp_defined_paren,
    UO_sp_throw_paren,
+   UO_sp_after_throw,
    UO_sp_catch_paren,
    UO_sp_version_paren,
    UO_sp_scope_paren,
@@ -265,13 +280,14 @@ enum uncrustify_options
    UO_sp_before_unnamed_ptr_star,
    UO_sp_after_ptr_star,        // space after a '*' that is part of a type
    UO_sp_after_ptr_star_func,   // space between a '*' and a function proto/def
+   UO_sp_ptr_star_paren,
    UO_sp_before_ptr_star_func,
    UO_sp_after_byref_func,
    UO_sp_before_byref_func,
    UO_sp_between_ptr_star,      // space between two '*' that are part of a type
 
-   UO_sp_special_semi,      /* space empty stmt ';' on while, if, for
-                             * example "while (*p++ = ' ') ;" */
+   UO_sp_special_semi,          /* space empty stmt ';' on while, if, for
+                                 * example "while (*p++ = ' ') ;" */
    UO_sp_before_semi,           // space before all ';'
    UO_sp_before_semi_for,       // space before the two ';' in a for() - non-empty
    UO_sp_before_semi_for_empty, // space before ';' in empty for statement
@@ -316,6 +332,8 @@ enum uncrustify_options
    UO_sp_after_oc_scope,
    UO_sp_before_oc_colon,
    UO_sp_after_oc_colon,
+   UO_sp_before_oc_dict_colon,
+   UO_sp_after_oc_dict_colon,
    UO_sp_before_send_oc_colon,
    UO_sp_after_send_oc_colon,
    UO_sp_after_oc_type,
@@ -325,6 +343,8 @@ enum uncrustify_options
    UO_sp_inside_oc_at_sel_parens,
    UO_sp_before_oc_block_caret,
    UO_sp_after_oc_block_caret,
+   UO_sp_after_oc_msg_receiver,
+   UO_sp_after_oc_property,
    UO_sp_cond_colon,
    UO_sp_cond_question,
    UO_sp_case_label,
@@ -332,8 +352,12 @@ enum uncrustify_options
    UO_sp_cmt_cpp_start,
    UO_sp_endif_cmt,
    UO_sp_after_new,
-   UO_sp_before_tr_emb_cmt,  // treatment of spaces before comments following code
-   UO_sp_num_before_tr_emb_cmt,  // number of spaces before comments following code
+   UO_sp_before_tr_emb_cmt,     // treatment of spaces before comments following code
+   UO_sp_num_before_tr_emb_cmt, // number of spaces before comments following code
+   UO_sp_annotation_paren,
+   UO_sp_after_for_colon,
+   UO_sp_before_for_colon,
+   UO_sp_extern_paren,
 
    /*
     * Line splitting options (for long lines)
@@ -342,7 +366,8 @@ enum uncrustify_options
    UO_code_width,           // ie 80 columns
    UO_ls_for_split_full,    // try to split long 'for' statements at semi-colons
    UO_ls_func_split_full,   // try to split long func proto/def at comma
-   //UO_ls_before_bool_op,    //TODO: break line before of after boolean op
+   UO_ls_code_width,        // try to split at code_width
+   //UO_ls_before_bool_op,    //TODO: break line before or after boolean op
    //UO_ls_before_paren,      //TODO: break before open paren
    //UO_ls_after_arith,       //TODO: break after arith op '+', etc
    //UO_ls_honor_newlines,    //TODO: don't remove newlines on split lines
@@ -378,6 +403,7 @@ enum uncrustify_options
    UO_align_var_struct_span,      // span for struct/union (0=don't align)
    UO_align_var_struct_thresh,    // threshold for struct/union, 0=no limit
    UO_align_var_struct_gap,       // gap for struct/union
+   UO_align_pp_define_together,   // align macro functions and variables together
    UO_align_pp_define_span,       // align bodies in #define statements
    //UO_align_pp_define_col_min,    //TODO: min column for a #define value
    //UO_align_pp_define_col_max,    //TODO: max column for a #define value
@@ -406,6 +432,7 @@ enum uncrustify_options
    UO_align_left_shift,
 //    UO_align_oc_msg_colon,
    UO_align_oc_msg_colon_span,
+   UO_align_oc_msg_colon_first,
    UO_align_oc_decl_colon,
 
 
@@ -432,11 +459,20 @@ enum uncrustify_options
    UO_nl_func_type_name_class,       // newline between return type and func name in class
    UO_nl_func_scope_name,
    UO_nl_func_proto_type_name,       // nl_func_type_name, but for prottypes
-   UO_nl_func_var_def_blk,           // newline after a block of variable defs
+   UO_nl_func_var_def_blk,           // newline after first block of func variable defs
+   UO_nl_typedef_blk_start,          // newline before typedef block
+   UO_nl_typedef_blk_end,            // newline after typedef block
+   UO_nl_typedef_blk_in,             // newline max within typedef block
+   UO_nl_var_def_blk_start,          // newline before variable defs block
+   UO_nl_var_def_blk_end,            // newline after variable defs block
+   UO_nl_var_def_blk_in,             // newline max within variable defs block
    UO_nl_before_case,                // newline before 'case' statement
    UO_nl_before_throw,
+   UO_nl_before_return,
    UO_nl_after_return,               /* newline after return statement */
    UO_nl_return_expr,
+   UO_nl_after_annotation,
+   UO_nl_between_annotation,
    UO_nl_after_case,                 /* disallow nested "case 1: a=3;" */
    UO_nl_after_semicolon,            // disallow multiple statements on a line "a=1;b=4;"
    UO_nl_after_brace_open,           // force a newline after a brace open
@@ -445,6 +481,7 @@ enum uncrustify_options
    UO_nl_after_vbrace_open_empty,    // force a newline after a virtual brace open
    UO_nl_after_brace_close,          // force a newline after a brace close
    UO_nl_after_vbrace_close,         // force a newline after a virtual brace close
+   UO_nl_brace_struct_var,           // force a newline after a brace close
    UO_nl_fcall_brace,                /* newline between function call and open brace */
    UO_nl_squeeze_ifdef,              /* no blanks after #ifxx, #elxx, or before #endif */
    UO_nl_enum_brace,                 /* nl between enum and brace */
@@ -469,6 +506,9 @@ enum uncrustify_options
    UO_nl_catch_brace,                /* nl between catch and { */
    UO_nl_brace_catch,                /* nl between } and catch */
    UO_nl_while_brace,                /* nl between while and { */
+   UO_nl_unittest_brace,             /* nl between unittest and { */
+   UO_nl_scope_brace,
+   UO_nl_version_brace,
    UO_nl_using_brace,
    UO_nl_switch_brace,               /* nl between switch and { */
    UO_nl_brace_else,                 // nl between } and else
@@ -515,6 +555,9 @@ enum uncrustify_options
    UO_nl_create_for_one_liner,
    UO_nl_create_while_one_liner,
 
+   UO_nl_oc_msg_args,
+   UO_nl_oc_msg_leave_one_liner,
+
    UO_pos_arith,                  // position of trailing/leading arithmetic ops
    UO_pos_assign,                 // position of trailing/leading =
    UO_pos_bool,                   // position of trailing/leading &&/||
@@ -538,7 +581,6 @@ enum uncrustify_options
    UO_nl_after_func_body_one_liner,  // after the closing brace of a single line function body
    UO_nl_after_func_proto,           // after each prototype
    UO_nl_after_func_proto_group,     // after a block of prototypes
-   //UO_nl_after_var_def_group,        // after a group of variable defs at top of proc
    //UO_nl_after_ifdef,                // after #if or #ifdef - but not if covers whole file
    UO_nl_after_struct,
    UO_nl_after_class,
@@ -553,6 +595,7 @@ enum uncrustify_options
 
    UO_eat_blanks_after_open_brace,   // remove blank lines after {
    UO_eat_blanks_before_close_brace, // remove blank lines before }
+   UO_nl_remove_extra_newlines,      // remove extra nl aggressiveness
 
 
    /*
@@ -600,7 +643,7 @@ enum uncrustify_options
    UO_cmt_c_nl_end,             // put a nl before the */ in a combined group
    UO_cmt_cpp_nl_start,         // put a blank /* at the start of a converted group
    UO_cmt_cpp_nl_end,           // put a nl before the */ in a converted group
-   UO_cmt_multi_check_last,
+   UO_cmt_multi_check_last,     // no space after '*' prefix when comment start and end are of equal length
 
    UO_cmt_insert_file_header,
    UO_cmt_insert_file_footer,
@@ -637,7 +680,7 @@ struct option_map_value
 };
 
 
-typedef map<string, option_map_value>::iterator        option_name_map_it;
+typedef map<string, option_map_value>::iterator             option_name_map_it;
 typedef map<uncrustify_groups, group_map_value>::iterator   group_map_it;
 typedef list<uncrustify_options>::iterator                  option_list_it;
 typedef list<uncrustify_options>::const_iterator            option_list_cit;
