@@ -6,6 +6,7 @@
  * @license GPL v2+
  */
 #include "uncrustify_types.h"
+#include "uncrustify_version.h"
 #include "prototypes.h"
 #include "chunk_list.h"
 #include "unc_ctype.h"
@@ -33,6 +34,9 @@ static chunk_t *output_comment_cpp(chunk_t *pc);
 static void add_comment_text(const unc_text& text,
                              cmt_reflow& cmt, bool esc_close);
 
+#define LOG_CONTTEXT() \
+   LOG_FMT(LCONTTEXT, "%s:%d set cont_text to '%s'\n", __func__, __LINE__, cmt.cont_text.c_str())
+
 /**
  * All output text is sent here, one char at a time.
  */
@@ -223,15 +227,13 @@ void output_parsed(FILE *pfile)
    chunk_t *pc;
    int     cnt;
 
-   output_options(pfile);
-   output_defines(pfile);
-   output_types(pfile);
+   save_option_file(pfile, false);
 
-   fprintf(pfile, "-=====-\n");
-   fprintf(pfile, "Line      Tag          Parent     Columns  Br/Lvl/pp Flag Nl  Text");
+   fprintf(pfile, "# -=====-\n");
+   fprintf(pfile, "# Line      Tag          Parent     Columns  Br/Lvl/pp Flag Nl  Text");
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
    {
-      fprintf(pfile, "\n%3d> %13.13s[%13.13s][%2d/%2d/%2d][%d/%d/%d][%10" PRIx64 "][%d-%d]",
+      fprintf(pfile, "\n# %3d> %13.13s[%13.13s][%2d/%2d/%2d][%d/%d/%d][%10" PRIx64 "][%d-%d]",
               pc->orig_line, get_token_name(pc->type),
               get_token_name(pc->parent_type),
               pc->column, pc->orig_col, pc->orig_col_end,
@@ -254,39 +256,11 @@ void output_parsed(FILE *pfile)
          }
       }
    }
-   fprintf(pfile, "\n-=====-\n");
+   fprintf(pfile, "\n# -=====-\n");
    fflush(pfile);
 }
 
 
-void output_options(FILE *pfile)
-{
-   int idx;
-   const option_map_value *ptr;
-
-   fprintf(pfile, "-== Options ==-\n");
-   for (idx = 0; idx < UO_option_count; idx++)
-   {
-      ptr = get_option_name(idx);
-      if (ptr != NULL)
-      {
-         if (ptr->type == AT_STRING)
-         {
-            fprintf(pfile, "%3d) %32s = \"%s\"\n",
-                    ptr->id, ptr->name,
-                    op_val_to_string(ptr->type, cpd.settings[ptr->id]).c_str());
-         }
-         else
-         {
-            fprintf(pfile, "%3d) %32s = %s\n",
-                    ptr->id, ptr->name,
-                    op_val_to_string(ptr->type, cpd.settings[ptr->id]).c_str());
-         }
-      }
-   }
-}
-
-
 /**
  * This renders the chunk list to a file.
  */
@@ -361,8 +335,12 @@ void output_text(FILE *pfile)
                   }
                }
             }
+            output_to_column(pc->column, false);
+         }
+         else
+         {
+            output_to_column(pc->column, (cpd.settings[UO_indent_with_tabs].n == 2));
          }
-         output_to_column(pc->column, (cpd.settings[UO_indent_with_tabs].n == 2));
          add_char('\\');
          add_char('\n');
          cpd.did_newline = 1;
@@ -431,7 +409,7 @@ void output_text(FILE *pfile)
                          (chunk_is_comment(pc) &&
                           (cpd.settings[UO_indent_with_tabs].n != 0));
 
-            LOG_FMT(LOUTIND, "  %d> col %d/%d - ", pc->orig_line, pc->column, cpd.column);
+            LOG_FMT(LOUTIND, "  %d> col %d/%d/%d - ", pc->orig_line, pc->column, pc->column_indent, cpd.column);
          }
          else
          {
@@ -507,6 +485,10 @@ static int cmt_parse_lead(const unc_text& line, int is_last)
    {
       return len;
    }
+   if ((len == 1) && (line[0] == '*'))
+   {
+      return len;
+   }
    if (is_last && (len > 0))
    {
       return len;
@@ -622,7 +604,10 @@ static void calculate_comment_body_indent(cmt_reflow &cmt, const unc_text& str)
       }
       else
       {
-         width = 0;
+         if ((width != 1) || (str[idx - 1] != '*'))
+         {
+            width = 0;
+         }
          break;
       }
    }
@@ -932,6 +917,8 @@ static void add_comment_text(const unc_text& text,
             add_char(' ');
          }
          add_text(cmt.cont_text);
+         output_to_column(cmt.column + cpd.settings[UO_cmt_sp_after_star_cont].n,
+                          false);
       }
       else
       {
@@ -996,6 +983,10 @@ static void output_cmt_start(cmt_reflow& cmt, chunk_t *pc)
       chunk_t *prev = chunk_get_prev(pc);
       if (prev != NULL)
       {
+         while (prev->prev && prev->type == CT_VBRACE_CLOSE)
+         {
+            prev = prev->prev;
+         }
          int col_min = prev->column + prev->len() + 1;
          if (cmt.column < col_min)
          {
@@ -1074,6 +1065,7 @@ static chunk_t *output_comment_c(chunk_t *first)
    cmt.reflow = (cpd.settings[UO_cmt_reflow_mode].n != 1);
 
    cmt.cont_text = cpd.settings[UO_cmt_star_cont].b ? " *" : "  ";
+   LOG_CONTTEXT();
 
    /* See if we can combine this comment with the next comment */
    if (!cpd.settings[UO_cmt_c_group].b ||
@@ -1103,6 +1095,7 @@ static chunk_t *output_comment_c(chunk_t *first)
    if (cpd.settings[UO_cmt_c_nl_end].b)
    {
       cmt.cont_text = " ";
+      LOG_CONTTEXT();
       add_comment_text("\n", cmt, false);
    }
    add_comment_text("*/", cmt, false);
@@ -1127,7 +1120,8 @@ static chunk_t *output_comment_cpp(chunk_t *first)
    /* CPP comments can't be grouped unless they are converted to C comments */
    if (!cpd.settings[UO_cmt_cpp_to_c].b)
    {
-      cmt.cont_text = (cpd.settings[UO_sp_cmt_cpp_start].a & AV_REMOVE) ? "//" : "// ";
+      cmt.cont_text = (cpd.settings[UO_sp_cmt_cpp_start].a == AV_REMOVE) ? "//" : "// ";
+      LOG_CONTTEXT();
 
       if (cpd.settings[UO_sp_cmt_cpp_start].a == AV_IGNORE)
       {
@@ -1151,7 +1145,7 @@ static chunk_t *output_comment_cpp(chunk_t *first)
          {
             if (cpd.settings[UO_sp_cmt_cpp_start].a & AV_ADD)
             {
-               if (!unc_isspace(tmp[0]))
+               if (!unc_isspace(tmp[0]) && (tmp[0] != '/'))
                {
                   add_comment_text(" ", cmt, false);
                }
@@ -1165,6 +1159,7 @@ static chunk_t *output_comment_cpp(chunk_t *first)
 
    /* We are going to convert the CPP comments to C comments */
    cmt.cont_text = cpd.settings[UO_cmt_star_cont].b ? " * " : "   ";
+   LOG_CONTTEXT();
 
    /* See if we can combine this comment with the next comment */
    if (!cpd.settings[UO_cmt_cpp_group].b ||
@@ -1208,6 +1203,7 @@ static chunk_t *output_comment_cpp(chunk_t *first)
    if (cpd.settings[UO_cmt_cpp_nl_end].b)
    {
       cmt.cont_text = "";
+      LOG_CONTTEXT();
       add_comment_text("\n", cmt, false);
    }
    add_comment_text(" */", cmt, false);
@@ -1258,7 +1254,6 @@ static void output_comment_multi(chunk_t *pc)
    int        cmt_col;
    int        cmt_idx;
    int        ch;
-   chunk_t    *prev;
    unc_text   line;
    int        line_count = 0;
    int        ccol; /* the col of subsequent comment lines */
@@ -1268,7 +1263,6 @@ static void output_comment_multi(chunk_t *pc)
 
    //LOG_FMT(LSYS, "%s: line %d\n", __func__, pc->orig_line);
 
-   prev = chunk_get_prev(pc);
    output_cmt_start(cmt, pc);
    cmt.reflow = (cpd.settings[UO_cmt_reflow_mode].n != 1);
 
@@ -1279,6 +1273,7 @@ static void output_comment_multi(chunk_t *pc)
 
    cmt.cont_text = !cpd.settings[UO_cmt_indent_multi].b ? "" :
                    (cpd.settings[UO_cmt_star_cont].b ? "* " : "  ");
+   LOG_CONTTEXT();
 
    //LOG_FMT(LSYS, "Indenting1 line %d to col %d (orig=%d) col_diff=%d xtra=%d cont='%s'\n",
    //        pc->orig_line, cmt_col, pc->orig_col, col_diff, cmt.xtra_indent, cmt.cont_text.c_str());
@@ -1382,7 +1377,7 @@ static void output_comment_multi(chunk_t *pc)
           *   something clearly non-alphanumeric (you see where we're going with this?)
           *
           * - bullet lists that are closely spaced: bullets are always non-alphanumeric
-          *   characters, such as '-' or '+' (or, oh horor, '*' - that's bloody ambiguous
+          *   characters, such as '-' or '+' (or, oh horror, '*' - that's bloody ambiguous
           *   to parse :-( ... with or without '*' comment start prefix, that's the
           *   question, then.)
           *
@@ -1516,6 +1511,11 @@ static void output_comment_multi(chunk_t *pc)
                   if (idx > 0)
                   {
                      cmt.cont_text.set(line, 0, idx);
+                     LOG_CONTTEXT();
+                     if ((line.size() >= 2) && (line[0] == '*') && unc_isalnum(line[1]))
+                     {
+                        line.insert(1, ' ');
+                     }
                   }
                   else
                   {
@@ -1544,9 +1544,8 @@ static void output_comment_multi(chunk_t *pc)
  */
 static void output_comment_multi_simple(chunk_t *pc)
 {
-   int        cmt_col = pc->column;
    int        cmt_idx;
-   char       ch;
+   int        ch;
    int        line_count = 0;
    int        ccol;
    int        col_diff = 0;
@@ -1559,13 +1558,11 @@ static void output_comment_multi_simple(chunk_t *pc)
    if (chunk_is_newline(chunk_get_prev(pc)))
    {
       /* The comment should be indented correctly */
-      cmt_col  = pc->column;
       col_diff = pc->orig_col - pc->column;
    }
    else
    {
       /* The comment starts after something else */
-      cmt_col  = pc->orig_col;
       col_diff = 0;
    }
 
