@@ -31,10 +31,11 @@ static void remove_semicolon(chunk_t *pc)
 
 /**
  * Removes superfluous semicolons:
- *  - after brace close whose parent is IF, ELSE, SWITCH, WHILE, FOR
+ *  - after brace close whose parent is IF, ELSE, SWITCH, WHILE, FOR, NAMESPACE
  *  - after another semicolon where parent is not FOR
  *  - (D) after brace close whose parent is ENUM/STRUCT/UNION
  *  - after an open brace
+ *  - when not in a #DEFINE
  */
 void remove_extra_semicolons(void)
 {
@@ -47,7 +48,7 @@ void remove_extra_semicolons(void)
    {
       next = chunk_get_next_ncnl(pc);
 
-      if ((pc->type == CT_SEMICOLON) &&
+      if ((pc->type == CT_SEMICOLON) && !(pc->flags & PCF_IN_PREPROC) &&
           ((prev = chunk_get_prev_ncnl(pc)) != NULL))
       {
          LOG_FMT(LSCANSEMI, "Semi on %d:%d, prev = '%s' [%s/%s]\n",
@@ -64,7 +65,8 @@ void remove_extra_semicolons(void)
               (prev->parent_type == CT_FOR) ||
               (prev->parent_type == CT_FUNC_DEF) ||
               (prev->parent_type == CT_OC_MSG_DECL) ||
-              (prev->parent_type == CT_FUNC_CLASS)))
+              (prev->parent_type == CT_FUNC_CLASS) ||
+              (prev->parent_type == CT_NAMESPACE)))
          {
             remove_semicolon(pc);
          }
