@@ -31,6 +31,7 @@ struct no_space_table_s
  */
 struct no_space_table_s no_space_table[] =
 {
+   { CT_OC_AT,          CT_UNKNOWN       },
    { CT_INCDEC_BEFORE,  CT_WORD          },
    { CT_UNKNOWN,        CT_INCDEC_AFTER  },
    { CT_UNKNOWN,        CT_LABEL_COLON   },
@@ -118,6 +119,12 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       log_rule("sp_pp_stringify");
       return(cpd.settings[UO_sp_pp_stringify].a);
    }
+   if ((second->type == CT_POUND) && (second->flags & PCF_IN_PREPROC) &&
+       (first->parent_type != CT_MACRO_FUNC))
+   {
+      log_rule("sp_before_pp_stringify");
+      return(cpd.settings[UO_sp_before_pp_stringify].a);
+   }
 
    if ((first->type == CT_SPACE) || (second->type == CT_SPACE))
    {
@@ -170,6 +177,17 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(argval_t(cpd.settings[UO_sp_case_label].a | AV_ADD));
    }
 
+   if (first->type == CT_FOR_COLON)
+   {
+      log_rule("sp_after_for_colon");
+      return cpd.settings[UO_sp_after_for_colon].a;
+   }
+   if (second->type == CT_FOR_COLON)
+   {
+      log_rule("sp_before_for_colon");
+      return cpd.settings[UO_sp_before_for_colon].a;
+   }
+
    if ((first->type == CT_QUESTION) || (second->type == CT_QUESTION))
    {
       if (cpd.settings[UO_sp_cond_question].a != AV_IGNORE)
@@ -295,6 +313,7 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       /* Let the comment spacing rules handle this */
    }
 
+   /* puts a space in the rare '+-' or '-+' */
    if (((first->type == CT_NEG) || (first->type == CT_POS) || (first->type == CT_ARITH)) &&
        ((second->type == CT_NEG) || (second->type == CT_POS) || (second->type == CT_ARITH)))
    {
@@ -442,15 +461,30 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(cpd.settings[UO_sp_assign].a);
    }
 
-   if (second->type == CT_OC_BLOCK_CARET)
+   // Handle the special lambda case for C++11:
+   //    [=](Something arg){.....}
+   if ((cpd.settings[UO_sp_cpp_lambda_assign].a != AV_IGNORE) &&
+       (((first->type == CT_SQUARE_OPEN) &&
+         (first->parent_type == CT_CPP_LAMBDA) &&
+         (second->type == CT_ASSIGN))
+        ||
+        ((first->type == CT_ASSIGN) &&
+         (second->type == CT_SQUARE_CLOSE) &&
+         (second->parent_type == CT_CPP_LAMBDA))))
    {
-      log_rule("sp_before_oc_block_caret");
-      return(cpd.settings[UO_sp_before_oc_block_caret].a);
+      log_rule("UO_sp_cpp_lambda_assign");
+      return(cpd.settings[UO_sp_cpp_lambda_assign].a);
    }
-   if (first->type == CT_OC_BLOCK_CARET)
+
+   // Handle the special lambda case for C++11:
+   //    [](Something arg){.....}
+   if ((cpd.settings[UO_sp_cpp_lambda_paren].a != AV_IGNORE) &&
+       (first->type == CT_SQUARE_CLOSE) &&
+       (first->parent_type == CT_CPP_LAMBDA) &&
+       (second->type == CT_FPAREN_OPEN))
    {
-      log_rule("sp_after_oc_block_caret");
-      return(cpd.settings[UO_sp_after_oc_block_caret].a);
+      log_rule("UO_sp_cpp_lambda_paren");
+      return(cpd.settings[UO_sp_cpp_lambda_paren].a);
    }
 
    if (second->type == CT_ASSIGN)
@@ -507,6 +541,22 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(cpd.settings[UO_sp_assign].a);
    }
 
+   if (second->type == CT_OC_BLOCK_CARET)
+   {
+      log_rule("sp_before_oc_block_caret");
+      return(cpd.settings[UO_sp_before_oc_block_caret].a);
+   }
+   if (first->type == CT_OC_BLOCK_CARET)
+   {
+      log_rule("sp_after_oc_block_caret");
+      return(cpd.settings[UO_sp_after_oc_block_caret].a);
+   }
+   if (second->type == CT_OC_MSG_FUNC)
+   {
+      log_rule("sp_after_oc_msg_receiver");
+      return(cpd.settings[UO_sp_after_oc_msg_receiver].a);
+   }
+
    /* "a [x]" vs "a[x]" */
    if ((second->type == CT_SQUARE_OPEN) && (second->parent_type != CT_OC_MSG))
    {
@@ -695,6 +745,13 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(AV_REMOVE);  /* TODO: make this configurable? */
    }
 
+   /* handle the space between parens in fcn type 'void (*f)(void)' */
+   if (first->type == CT_TPAREN_CLOSE)
+   {
+      log_rule("sp_after_tparen_close");
+      return(cpd.settings[UO_sp_after_tparen_close].a);
+   }
+
    if ((first->type == CT_FUNC_PROTO) ||
        ((second->type == CT_FPAREN_OPEN) &&
         (second->parent_type == CT_FUNC_PROTO)))
@@ -748,10 +805,15 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(cpd.settings[UO_sp_defined_paren].a);
    }
 
-   if ((first->type == CT_THROW) && (second->type == CT_PAREN_OPEN))
+   if (first->type == CT_THROW)
    {
-      log_rule("sp_throw_paren");
-      return(cpd.settings[UO_sp_throw_paren].a);
+      if (second->type == CT_PAREN_OPEN)
+      {
+         log_rule("sp_throw_paren");
+         return(cpd.settings[UO_sp_throw_paren].a);
+      }
+      log_rule("sp_after_throw");
+      return(cpd.settings[UO_sp_after_throw].a);
    }
 
    if ((first->type == CT_THIS) && (second->type == CT_PAREN_OPEN))
@@ -860,7 +922,7 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       }
 
       /* Arith after a cast comes first */
-      if (second->type == CT_ARITH)
+      if ((second->type == CT_ARITH) || (second->type == CT_CARET))
       {
          log_rule("sp_arith");
          return(cpd.settings[UO_sp_arith].a);
@@ -900,9 +962,16 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(cpd.settings[UO_sp_inside_fparen].a);
    }
 
+   /* "foo(...)" vs "foo( ... )" */
+   if ((first->type == CT_TPAREN_OPEN) || (second->type == CT_TPAREN_CLOSE))
+   {
+      log_rule("sp_inside_tparen");
+      return(cpd.settings[UO_sp_inside_tparen].a);
+   }
+
    if (first->type == CT_PAREN_CLOSE)
    {
-      if (first->parent_type == CT_OC_RTYPE)
+      if (first->flags & PCF_OC_RTYPE) //== CT_OC_RTYPE)
       {
          log_rule("sp_after_oc_return_type");
          return(cpd.settings[UO_sp_after_oc_return_type].a);
@@ -992,6 +1061,12 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       log_rule("sp_inside_sparen_close");
       return(cpd.settings[UO_sp_inside_sparen_close].a);
    }
+   if ((first->type == CT_SPAREN_OPEN) &&
+       (cpd.settings[UO_sp_inside_sparen_open].a != AV_IGNORE))
+   {
+      log_rule("sp_inside_sparen_open");
+      return(cpd.settings[UO_sp_inside_sparen_open].a);
+   }
    if ((first->type == CT_SPAREN_OPEN) || (second->type == CT_SPAREN_CLOSE))
    {
       log_rule("sp_inside_sparen");
@@ -1001,6 +1076,8 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
    if ((cpd.settings[UO_sp_after_class_colon].a != AV_IGNORE) &&
        (first->type == CT_CLASS_COLON))
    {
+      min_sp = cpd.settings[UO_indent_ctor_init_leading].n - 1; // default indent is 1 space
+
       log_rule("sp_after_class_colon");
       return(cpd.settings[UO_sp_after_class_colon].a);
    }
@@ -1028,7 +1105,8 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(AV_ADD);
    }
 
-   if ((first->type == CT_ARITH) || (second->type == CT_ARITH))
+   if ((first->type == CT_ARITH) || (first->type == CT_CARET) ||
+       (second->type == CT_ARITH) || (second->type == CT_CARET))
    {
       log_rule("sp_arith");
       return(cpd.settings[UO_sp_arith].a);
@@ -1057,6 +1135,15 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(AV_REMOVE);
    }
 
+   if ((first->type == CT_PTR_TYPE) &&
+       (cpd.settings[UO_sp_ptr_star_paren].a != AV_IGNORE) &&
+       ((second->type == CT_FPAREN_OPEN) ||
+        (second->type == CT_TPAREN_OPEN)))
+   {
+      log_rule("sp_ptr_star_paren");
+      return cpd.settings[UO_sp_ptr_star_paren].a;
+   }
+
    if ((first->type == CT_PTR_TYPE) &&
        (second->type == CT_PTR_TYPE) &&
        (cpd.settings[UO_sp_between_ptr_star].a != AV_IGNORE))
@@ -1068,7 +1155,8 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
    if ((first->type == CT_PTR_TYPE) &&
        (cpd.settings[UO_sp_after_ptr_star_func].a != AV_IGNORE) &&
        ((first->parent_type == CT_FUNC_DEF) ||
-        (first->parent_type == CT_FUNC_PROTO)))
+        (first->parent_type == CT_FUNC_PROTO) ||
+        (first->parent_type == CT_FUNC_VAR)))
    {
       log_rule("sp_after_ptr_star_func");
       return(cpd.settings[UO_sp_after_ptr_star_func].a);
@@ -1218,6 +1306,12 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(cpd.settings[UO_sp_before_sparen].a);
    }
 
+   if ((second->type == CT_PAREN_OPEN) && (second->parent_type == CT_TEMPLATE))
+   {
+      log_rule("UO_sp_before_template_paren");
+      return(cpd.settings[UO_sp_before_template_paren].a);
+   }
+
    if ((second->type != CT_PTR_TYPE) &&
        ((first->type == CT_QUALIFIER) || (first->type == CT_TYPE)))
    {
@@ -1291,6 +1385,16 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       log_rule("sp_after_oc_scope");
       return(cpd.settings[UO_sp_after_oc_scope].a);
    }
+   if (first->type == CT_OC_DICT_COLON)
+   {
+      log_rule("sp_after_oc_dict_colon");
+      return(cpd.settings[UO_sp_after_oc_dict_colon].a);
+   }
+   if (second->type == CT_OC_DICT_COLON)
+   {
+      log_rule("sp_before_oc_dict_colon");
+      return(cpd.settings[UO_sp_before_oc_dict_colon].a);
+   }
    if (first->type == CT_OC_COLON)
    {
       if (first->flags & PCF_IN_OC_MSG)
@@ -1346,6 +1450,24 @@ static argval_t do_space(chunk_t *first, chunk_t *second, int& min_sp, bool comp
       return(cpd.settings[UO_sp_after_new].a);
    }
 
+   if ((first->type == CT_ANNOTATION) && chunk_is_paren_open(second))
+   {
+      log_rule("sp_annotation_paren");
+      return(cpd.settings[UO_sp_annotation_paren].a);
+   }
+
+   if (first->type == CT_OC_PROPERTY)
+   {
+      log_rule("sp_after_oc_property");
+      return(cpd.settings[UO_sp_after_oc_property].a);
+   }
+
+   if ((first->type == CT_EXTERN) && (second->type == CT_PAREN_OPEN))
+   {
+      log_rule("sp_extern_paren");
+      return cpd.settings[UO_sp_extern_paren].a;
+   }
+
    for (idx = 0; idx < (int)ARRAY_SIZE(no_space_table); idx++)
    {
       if (((no_space_table[idx].first == CT_UNKNOWN) ||
@@ -1420,7 +1542,8 @@ void space_text(void)
          pc->flags &= ~PCF_FORCE_SPACE;
          if ((pc->len() > 0) &&
              !chunk_is_str(pc, "[]", 2) &&
-             !chunk_is_str(pc, "()", 2))
+             !chunk_is_str(pc, "()", 2) &&
+             !pc->str.startswith("@\""))
          {
             /* Find the next non-empty chunk on this line */
             tmp = next;
@@ -1450,7 +1573,25 @@ void space_text(void)
                   if ((ct != NULL) && ((int)strlen(ct->tag) != pc->len()))
                   {
                      /* punctuator parsed to a different size.. */
-                     pc->flags |= PCF_FORCE_SPACE;
+
+                     /* C++11 allows '>>' to mean '> >' in templates:
+                      *   some_func<vector<string>>();
+                      */
+                     if ((cpd.lang_flags & (LANG_CPP | LANG_JAVA)) &&
+                         cpd.settings[UO_sp_permit_cpp11_shift].b &&
+                         (pc->type == CT_ANGLE_CLOSE) &&
+                         (next->type == CT_ANGLE_CLOSE))
+                     {
+                        /* allow '>' and '>' to become '>>' */
+                     }
+                     else if (strcmp(ct->tag, "[]") == 0)
+                     {
+                        /* this is OK */
+                     }
+                     else
+                     {
+                        pc->flags |= PCF_FORCE_SPACE;
+                     }
                   }
                }
             }
