@@ -79,6 +79,7 @@ typedef enum
    CT_SBOOL,               /* or, and */
    CT_ARITH,               /* +, -, /, <<, etc */
    CT_SARITH,              /* 'not', 'xor' */
+   CT_CARET,               /* ^ */
    CT_DEREF,               /* * dereference */
    CT_INCDEC_BEFORE,       /* ++a or --a */
    CT_INCDEC_AFTER,        /* a++ or a-- */
@@ -124,6 +125,7 @@ typedef enum
    CT_EXPORT,
    CT_FRIEND,
    CT_NAMESPACE,
+   CT_PACKAGE,
    CT_NEW,              /* may turn into CT_PBRACED if followed by a '(' */
    CT_OPERATOR,
    CT_OPERATOR_VAL,
@@ -165,13 +167,16 @@ typedef enum
    CT_FPAREN_OPEN,      /* 'function' paren after fcn/macro fcn */
    CT_FPAREN_CLOSE,
 
-   CT_BRACE_OPEN,
+   CT_TPAREN_OPEN,      /* 'type' paren used in function types */
+   CT_TPAREN_CLOSE,
+
+   CT_BRACE_OPEN,       /* {...} */
    CT_BRACE_CLOSE,
 
-   CT_VBRACE_OPEN,
+   CT_VBRACE_OPEN,      /* virtual brace, i.e. brace inserted by uncrustify */
    CT_VBRACE_CLOSE,
 
-   CT_SQUARE_OPEN,
+   CT_SQUARE_OPEN,      /* [...] */
    CT_SQUARE_CLOSE,
 
    CT_TSQUARE,          /* special case of [] */
@@ -180,17 +185,20 @@ typedef enum
    CT_MACRO_CLOSE,
    CT_MACRO_ELSE,
 
-   /* agregate types */
+   /* aggregate types */
    CT_LABEL,            /* a non-case label */
    CT_LABEL_COLON,      /* the colon for a label */
    CT_FUNCTION,         /* function - unspecified, call mark_function() */
    CT_FUNC_CALL,        /* function call */
    CT_FUNC_CALL_USER,   /* function call (special user) */
    CT_FUNC_DEF,         /* function definition/implementation */
+   CT_FUNC_TYPE,        /* function type - foo in "typedef void (*foo)(void)" */
+   CT_FUNC_VAR,         /* foo and parent type of first parens in "void (*foo)(void)" */
    CT_FUNC_PROTO,       /* function prototype */
    CT_FUNC_CLASS,       /* ctor or dtor for a class */
    CT_FUNC_CTOR_VAR,    /* variable or class initialization */
    CT_FUNC_WRAP,        /* macro that wraps the function name */
+   CT_PROTO_WRAP,       /* macro: "RETVAL PROTO_WRAP( fcn_name, (PARAMS))" */
    CT_MACRO_FUNC,       /* function-like macro */
    CT_MACRO,            /* a macro def */
    CT_QUALIFIER,        /* static, const, etc */
@@ -199,7 +207,8 @@ typedef enum
    CT_TYPE,
    CT_PTR_TYPE,         /* a '*' as part of a type */
    CT_TYPE_WRAP,        /* macro that wraps a type name */
-
+   CT_CPP_LAMBDA,       /* parent for '[=](...){...}' */
+   CT_CPP_LAMBDA_RET,   /* '->' in '[=](...) -> type {...}' */
    CT_BIT_COLON,        /* a ':' in a variable declaration */
 
    CT_OC_DYNAMIC,
@@ -219,14 +228,18 @@ typedef enum
    CT_OC_MSG_NAME,      /* ObjC: 'name' in  '[class func : param name: param];' (see also PCF_IN_OC_MSG) */
    CT_OC_MSG_SPEC,      /* ObjC: msg spec '-(void) func: (int) i;' */
    CT_OC_MSG_DECL,      /* ObjC: msg decl '-(void) func: (int) i { }' */
-   CT_OC_RTYPE,         /* ObjC: marks closing parens of the return type after scope */
+   CT_OC_RTYPE,         /* ObjC: marks parens of the return type after scope */
+   CT_OC_ATYPE,         /* ObjC: marks parens of the arg type after scope */
    CT_OC_COLON,         /* ObjC: the colon in a msg spec */
+   CT_OC_DICT_COLON,    /* ObjC: colon in dictionary constant: "KEY: VALUE" */
    CT_OC_SEL,           /* ObjC: @selector */
    CT_OC_SEL_NAME,      /* ObjC: selector name */
+   CT_OC_BLOCK,         /* ObjC: block parent type. */
    CT_OC_BLOCK_ARG,     /* ObjC: block arguments parent type. */
    CT_OC_BLOCK_TYPE,    /* ObjC: block declaration parent type, e.g. mainly the '(^block_t)' in 'void (^block_t)(int arg);' */
    CT_OC_BLOCK_EXPR,    /* ObjC: block expression with arg: '^(int arg) { arg++; };' and without (called a block literal): '^{ ... };' */
    CT_OC_BLOCK_CARET,   /* ObjC: block pointer caret: '^' */
+   CT_OC_AT,            /* ObjC: boxed constants using '@' */
 
    /* start PP types */
    CT_PP_DEFINE,        /* #define */
@@ -293,6 +306,8 @@ typedef enum
 
    /* Java */
    CT_ASSERT,           /* assert EXP1 [ : EXP2 ] ; */
+   CT_ANNOTATION,       /* @interface or @something(...) */
+   CT_FOR_COLON,        /* colon in "for ( TYPE var: EXPR ) { ... }" */
 } c_token_t;
 
 #endif   /* TOKEN_ENUM_H_INCLUDED */
