@@ -295,11 +295,11 @@ static bool parse_comment(tok_ctx& ctx, chunk_t& pc)
 
    /* account for opening two chars */
    pc.str = ctx.get();   /* opening '/' */
+   ch = ctx.get();
+   pc.str.append(ch);    /* second char */
 
-   ch = ctx.peek();
    if (ch == '/')
    {
-      pc.str.append(ctx.get());  /* store the '/' */
       pc.type = CT_COMMENT_CPP;
       while (true)
       {
@@ -349,7 +349,6 @@ static bool parse_comment(tok_ctx& ctx, chunk_t& pc)
    }
    else if (ch == '+')
    {
-      pc.str.append(ctx.get());  /* store the '+' */
       pc.type = CT_COMMENT;
       d_level++;
       while ((d_level > 0) && ctx.more())
@@ -379,7 +378,7 @@ static bool parse_comment(tok_ctx& ctx, chunk_t& pc)
 
             if (ch == '\r')
             {
-               if (ctx.peek(1) == '\n')
+               if (ctx.peek() == '\n')
                {
                   cpd.le_counts[LE_CRLF]++;
                   pc.str.append(ctx.get());  /* store the '\n' */
@@ -483,8 +482,17 @@ static void parse_suffix(tok_ctx& ctx, chunk_t& pc, bool forstring = false)
       int slen = 0;
       int oldsize = pc.str.size();
       tok_info ss;
-      ctx.save(ss);
 
+      /* don't add the suffix if we see L" or L' or S" */
+      int p1 = ctx.peek();
+      int p2 = ctx.peek(1);
+      if (forstring &&
+          (((p1 == 'L') && ((p2 == '"') || (p2 == '\''))) ||
+           ((p1 == 'S') && (p2 == '"'))))
+      {
+          return;
+      }
+      ctx.save(ss);
       while (ctx.more() && CharTable::IsKw2(ctx.peek()))
       {
          slen++;
@@ -750,6 +758,7 @@ static bool parse_string(tok_ctx& ctx, chunk_t& pc, int quote_idx, bool allow_es
       {
          pc.nl_count++;
          pc.type = CT_STRING_MULTI;
+         escaped = 0;
          continue;
       }
       if ((ch == '\r') && (ctx.peek() != '\n'))
@@ -757,6 +766,7 @@ static bool parse_string(tok_ctx& ctx, chunk_t& pc, int quote_idx, bool allow_es
          pc.str.append(ctx.get());
          pc.nl_count++;
          pc.type = CT_STRING_MULTI;
+         escaped = 0;
          continue;
       }
       if (!escaped)
@@ -947,8 +957,15 @@ bool parse_word(tok_ctx& ctx, chunk_t& pc, bool skipcheck)
    }
    else
    {
-      /* Turn it into a keyword now */
-      pc.type = find_keyword_type(pc.str.c_str(), pc.str.size());
+      if ((cpd.lang_flags & LANG_JAVA) && pc.str.startswith("@"))
+      {
+         pc.type = CT_ANNOTATION;
+      }
+      else
+      {
+         /* Turn it into a keyword now */
+         pc.type = find_keyword_type(pc.str.c_str(), pc.str.size());
+      }
    }
 
    return(true);
@@ -1270,16 +1287,6 @@ static bool parse_next(tok_ctx& ctx, chunk_t& pc)
       }
    }
 
-   /* Check for Obj-C NSString constants, ie @"hello" */
-   if (((cpd.lang_flags & LANG_OC) != 0) && (ctx.peek() == '@'))
-   {
-      if (ctx.peek(1) == '"')
-      {
-         parse_string(ctx, pc, 1, true);
-         return(true);
-      }
-   }
-
    /* handle C++0x strings u8"x" u"x" U"x" R"x" u8R"XXX(I'm a "raw UTF-8" string.)XXX" */
    ch = ctx.peek();
    if (((cpd.lang_flags & LANG_CPP) != 0) &&
@@ -1387,9 +1394,28 @@ static bool parse_next(tok_ctx& ctx, chunk_t& pc)
       }
    }
 
-   /* Check for pawn/ObjectiveC and normal identifiers */
+   /* Check for Objective C literals */
+   if ((cpd.lang_flags & LANG_OC) && (ctx.peek() == '@'))
+   {
+      int nc = ctx.peek(1);
+      if ((nc == '"') || (nc == '\''))
+      {
+         /* literal string */
+         parse_string(ctx, pc, 1, true);
+         return true;
+      }
+      else if ((nc >= '0') && (nc <= '9'))
+      {
+         /* literal number */
+         pc.str.append(ctx.get());  /* store the '@' */
+         parse_number(ctx, pc);
+         return true;
+      }
+   }
+
+   /* Check for pawn/ObjectiveC/Java and normal identifiers */
    if (CharTable::IsKw1(ctx.peek()) ||
-       ((ctx.peek() == '@') && CharTable::IsKw2(ctx.peek(1))))
+       ((ctx.peek() == '@') && CharTable::IsKw1(ctx.peek(1))))
    {
       parse_word(ctx, pc, false);
       return(true);
@@ -1441,7 +1467,6 @@ void tokenize(const deque<int>& data, chunk_t *ref)
    chunk_t            chunk;
    chunk_t            *pc    = NULL;
    chunk_t            *rprev = NULL;
-   chunk_t            *prev  = NULL;
    struct parse_frame frm;
    bool               last_was_tab = false;
 
@@ -1496,10 +1521,6 @@ void tokenize(const deque<int>& data, chunk_t *ref)
 
       /* Add the chunk to the list */
       rprev = pc;
-      if (!chunk_is_newline(pc) && !chunk_is_comment(pc))
-      {
-         prev = pc;
-      }
       if (rprev != NULL)
       {
          pc->flags |= rprev->flags & PCF_COPY_FLAGS;
