@@ -26,12 +26,12 @@ static chunk_t *handle_double_angle_close(chunk_t *pc)
 {
    chunk_t *next = chunk_get_next(pc);
 
-   if (next != NULL)
+   if (next)
    {
       if ((pc->type == CT_ANGLE_CLOSE) &&
           (next->type == CT_ANGLE_CLOSE) &&
           (pc->parent_type == CT_NONE) &&
-          (memcmp(pc->str, ">>", 2) == 0) &&
+          ((pc->orig_col_end + 1) == next->orig_col) &&
           (next->parent_type == CT_NONE))
       {
          pc->str.append('>');
@@ -107,7 +107,7 @@ void tokenize_cleanup(void)
           (pc->flags & PCF_IN_PREPROC) &&
           !chunk_get_next_ncnl(pc, CNAV_PREPROC))
       {
-         LOG_FMT(LWARN, "%s:%d Detected a macro that ends with a semicolon. Expect failures if used.\n",
+         LOG_FMT(LNOTE, "%s:%d Detected a macro that ends with a semicolon. Possible failures if used.\n",
                  cpd.filename, pc->orig_line);
       }
    }
@@ -267,7 +267,7 @@ void tokenize_cleanup(void)
          }
 
          /* handle "version(unittest) { }" vs "unittest { }" */
-         if ((pc->type == CT_UNITTEST) && (prev->type == CT_PAREN_OPEN))
+         if (prev && (pc->type == CT_UNITTEST) && (prev->type == CT_PAREN_OPEN))
          {
             pc->type = CT_WORD;
          }
@@ -299,6 +299,14 @@ void tokenize_cleanup(void)
          }
       }
 
+      /* Interface is only a keyword in MS land if followed by 'class' or 'struct'
+       * likewise, 'class' may be a member name in Java.
+       */
+      if ((pc->type == CT_CLASS) && !CharTable::IsKw1(next->str[0]))
+      {
+         pc->type = CT_WORD;
+      }
+
       /* Change item after operator (>=, ==, etc) to a CT_OPERATOR_VAL
        * Usually the next item is part of the operator.
        * In a few cases the next few tokens are part of it:
@@ -441,7 +449,7 @@ void tokenize_cleanup(void)
                {
                   break;
                }
-               if ((tmp->len() > 0) && isalpha(*tmp->str))
+               if ((tmp->len() > 0) && unc_isalpha(*tmp->str))
                {
                   tmp->type = CT_SQL_WORD;
                }
@@ -482,8 +490,8 @@ void tokenize_cleanup(void)
       if (cpd.lang_flags & LANG_OC)
       {
          if (((pc->type == CT_IF) ||
-             (pc->type == CT_FOR) ||
-             (pc->type == CT_WHILE)) &&
+              (pc->type == CT_FOR) ||
+              (pc->type == CT_WHILE)) &&
              !chunk_is_token(next, CT_PAREN_OPEN))
          {
             pc->type = CT_WORD;
@@ -578,21 +586,6 @@ void tokenize_cleanup(void)
          }
       }
 
-      /**
-       * Objective C @dynamic and @synthesize
-       *  @dynamic xxx, yyy;
-       *  @synthesize xxx, yyy;
-       * Just find the semicolon and mark it.
-       */
-      if (pc->type == CT_OC_DYNAMIC)
-      {
-         tmp = chunk_get_next_type(pc, CT_SEMICOLON, pc->level);
-         if (tmp != NULL)
-         {
-            tmp->parent_type = pc->type;
-         }
-      }
-
       /* Detect Objective C @property
        *  @property NSString *stringProperty;
        *  @property(nonatomic, retain) NSMutableDictionary *shareWith;
@@ -654,67 +647,6 @@ void tokenize_cleanup(void)
          }
       }
 
-      /* Mark Objective-C blocks (aka lambdas or closures)
-       *  The syntax and usage is exactly like C function pointers with two exceptions:
-       *  Instead of an asterisk they have a caret as pointer symbol.
-       *  In method declarations which take a block as parameter, there can be anonymous blocks, e.g.: (^)
-       *  1. block literal: ^{ ... };
-       *  2. block declaration: return_t (^name) (int arg1, int arg2, ...) NB: return_t is optional and name can be optional if found as param in a method declaration.
-       *  3. block expression: ^ return_t (int arg) { ... }; NB: return_t is optional
-       *
-       *  See http://developer.apple.com/mac/library/documentation/Cocoa/Conceptual/Blocks for more info...
-       */
-      if ((cpd.lang_flags & LANG_OC) &&
-          (pc->type == CT_ARITH) &&
-          (chunk_is_str(pc, "^", 1)) &&
-          (prev->type != CT_NUMBER) &&
-          (prev->type != CT_NUMBER_FP))
-      {
-         /* mark objc blocks caret so that we can process it later*/
-         pc->type = CT_OC_BLOCK_CARET;
-
-         if (prev->type == CT_PAREN_OPEN)
-         {
-            /* block declaration */
-            pc->parent_type = CT_OC_BLOCK_TYPE;
-         }
-         else if ((next->type == CT_PAREN_OPEN) ||
-                  (next->type == CT_BRACE_OPEN))
-         {
-            /* block expression without return type */
-            /* block literal */
-            pc->parent_type = CT_OC_BLOCK_EXPR;
-         }
-         else
-         {
-            /* block expression with return type (seldomly used) */
-            if (prev->type == CT_ASSIGN)
-            {
-               /* shortcut to spare the peeking below
-                * the XOR operator wouldn't be used directly
-                * after an assign all by itself */
-               pc->parent_type = CT_OC_BLOCK_EXPR;
-            }
-            else
-            {
-               /* this ones tricky because we don't know how many
-                * stars the return type has - if there even is one */
-               tmp = pc;
-               while ((tmp = chunk_get_next(tmp)) != NULL)
-               {
-                  /* we just peek ahead and see if the line contains
-                   * an open brace somewhere.
-                   * FIXME: this check needs to be more thorough. */
-                  if (tmp->type == CT_BRACE_OPEN)
-                  {
-                     pc->parent_type = CT_OC_BLOCK_EXPR;
-                     break;
-                  }
-               }
-            }
-         }
-      }
-
       /* Handle special preprocessor junk */
       if (pc->type == CT_PREPROC)
       {
@@ -876,6 +808,7 @@ static void check_template(chunk_t *start)
       /* A template requires a word/type right before the open angle */
       if ((prev->type != CT_WORD) &&
           (prev->type != CT_TYPE) &&
+          (prev->type != CT_COMMA) &&
           (prev->type != CT_OPERATOR_VAL) &&
           (prev->parent_type != CT_OPERATOR))
       {
@@ -925,7 +858,8 @@ static void check_template(chunk_t *start)
 
          if ((tokens[num_tokens - 1] == CT_ANGLE_OPEN) &&
              (pc->str[0] == '>') && (pc->len() > 1) &&
-             (cpd.settings[UO_tok_split_gte].b || chunk_is_str(pc, ">>", 2)))
+             (cpd.settings[UO_tok_split_gte].b ||
+              (chunk_is_str(pc, ">>", 2) && ((cpd.lang_flags & LANG_CPP) == 0))))
          {
             LOG_FMT(LTEMPL, " {split '%s' at %d:%d}",
                     pc->str.c_str(), pc->orig_line, pc->orig_col);
