@@ -25,6 +25,9 @@
 #include <cstring>
 #include <cerrno>
 #include <fcntl.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include "unc_ctype.h"
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
@@ -174,7 +177,7 @@ static void usage_exit(const char *msg, const char *argv0, int code)
            " -p FILE      : dump debug info to a file\n"
            " -L SEV       : Set the log severity (see log_levels.h)\n"
            " -s           : Show the log severity in the logs\n"
-           " --decode FLAG: Print FLAG (chunk flags) as text and exit\n"
+           " --decode     : decode remaining args (chunk flags) and exit\n"
            "\n"
            "Usage Examples\n"
            "cat foo.d | uncrustify -q -c my.cfg -l d\n"
@@ -226,7 +229,7 @@ static void redir_stdout(const char *output_file)
 
 int main(int argc, char *argv[])
 {
-   const char *cfg_file    = "uncrustify.cfg";
+   string     cfg_file;
    const char *parsed_file = NULL;
    const char *source_file = NULL;
    const char *output_file = NULL;
@@ -259,8 +262,8 @@ int main(int argc, char *argv[])
 
    if (arg.Present("--show-config"))
    {
-      print_options(stdout, true);
-      return(0);
+      print_options(stdout);
+      return EXIT_SUCCESS;
    }
 
 #ifdef WIN32
@@ -283,42 +286,51 @@ int main(int argc, char *argv[])
    }
    cpd.frag = arg.Present("--frag");
 
-   if ((p_arg = arg.Param("--decode")) != NULL)
+   if (arg.Present("--decode"))
    {
-      log_pcf_flags(LSYS, strtoul(p_arg, NULL, 16));
-      exit(EXIT_SUCCESS);
+      idx = 1;
+      while ((p_arg = arg.Unused(idx)) != NULL)
+      {
+         log_pcf_flags(LSYS, strtoul(p_arg, NULL, 16));
+      }
+      return EXIT_SUCCESS;
    }
 
    /* Get the config file name */
-   if (((cfg_file = arg.Param("--config")) == NULL) &&
-       ((cfg_file = arg.Param("-c")) == NULL))
+   if (((p_arg = arg.Param("--config")) != NULL) ||
+       ((p_arg = arg.Param("-c")) != NULL))
    {
-      /* Handled later */
+      cfg_file = p_arg;
    }
 
-#ifndef WIN32
-   /* Try to file a config at an alternate location */
-   char buf[512];
-   if (cfg_file == NULL)
+   /* Try to find a config file at an alternate location */
+   if (cfg_file.empty())
    {
-      cfg_file = getenv("UNCRUSTIFY_CONFIG");
-      if (cfg_file == NULL)
+      if (!unc_getenv("UNCRUSTIFY_CONFIG", cfg_file))
       {
-         const char *home = getenv("HOME");
+         string home;
 
-         if (home != NULL)
+         if (unc_homedir(home))
          {
             struct stat tmp_stat;
+            string      path;
 
-            snprintf(buf, sizeof(buf), "%s/.uncrustify.cfg", home);
-            if (stat(buf, &tmp_stat) == 0)
+            path = home + "/uncrustify.cfg";
+            if (stat(path.c_str(), &tmp_stat) == 0)
             {
-               cfg_file = buf;
+               cfg_file = path;
+            }
+            else
+            {
+               path = home + "/.uncrustify.cfg";
+               if (stat(path.c_str(), &tmp_stat) == 0)
+               {
+                  cfg_file = path;
+               }
             }
          }
       }
    }
-#endif
 
    /* Get the parsed file name */
    if (((parsed_file = arg.Param("--parsed")) != NULL) ||
@@ -404,6 +416,7 @@ int main(int argc, char *argv[])
    /* Grab the output override */
    output_file = arg.Param("-o");
 
+   LOG_FMT(LDATA, "config_file = %s\n", cfg_file.c_str());
    LOG_FMT(LDATA, "output_file = %s\n", (output_file != NULL) ? output_file : "null");
    LOG_FMT(LDATA, "source_file = %s\n", (source_file != NULL) ? source_file : "null");
    LOG_FMT(LDATA, "source_list = %s\n", (source_list != NULL) ? source_list : "null");
@@ -436,10 +449,10 @@ int main(int argc, char *argv[])
     * It is optional for "--universalindent" and "--detect", but required for
     * everything else.
     */
-   if (cfg_file != NULL)
+   if (!cfg_file.empty())
    {
-      cpd.filename = cfg_file;
-      if (load_option_file(cfg_file) < 0)
+      cpd.filename = cfg_file.c_str();
+      if (load_option_file(cpd.filename) < 0)
       {
          usage_exit("Unable to load the config file", argv[0], 56);
       }
@@ -456,13 +469,13 @@ int main(int argc, char *argv[])
          {
             fprintf(stderr, "Unable to open %s for write: %s (%d)\n",
                     output_file, strerror(errno), errno);
-            return(EXIT_FAILURE);
+            return EXIT_FAILURE;
          }
       }
 
       print_universal_indent_cfg(pfile);
 
-      return(EXIT_SUCCESS);
+      return EXIT_SUCCESS;
    }
 
    if (detect)
@@ -472,7 +485,7 @@ int main(int argc, char *argv[])
       if ((source_file == NULL) || (source_list != NULL))
       {
          fprintf(stderr, "The --detect option requires a single input file\n");
-         return(EXIT_FAILURE);
+         return EXIT_FAILURE;
       }
 
       /* Do some simple language detection based on the filename extension */
@@ -486,7 +499,7 @@ int main(int argc, char *argv[])
       {
          LOG_FMT(LERR, "Failed to load (%s)\n", source_file);
          cpd.error_count++;
-         return(EXIT_FAILURE);
+         return EXIT_FAILURE;
       }
 
       uncrustify_start(fm.data);
@@ -495,20 +508,16 @@ int main(int argc, char *argv[])
 
       redir_stdout(output_file);
       save_option_file(stdout, update_config_wd);
-      return(EXIT_SUCCESS);
+      return EXIT_SUCCESS;
    }
 
    /* Everything beyond this point requires a config file, so complain and
     * bail if we don't have one.
     */
-   if (cfg_file == NULL)
+   if (cfg_file.empty())
    {
-#ifdef WIN32
-      usage_exit("Specify the config file: -c file", argv[0], 58);
-#else
       usage_exit("Specify the config file with '-c file' or set UNCRUSTIFY_CONFIG",
                  argv[0], 58);
-#endif
    }
 
    /*
@@ -519,7 +528,7 @@ int main(int argc, char *argv[])
    {
       redir_stdout(output_file);
       save_option_file(stdout, update_config_wd);
-      return(0);
+      return EXIT_SUCCESS;
    }
 
    /* Check for unused args (ignore them) */
@@ -607,7 +616,7 @@ int main(int argc, char *argv[])
    clear_keyword_file();
    clear_defines();
 
-   return((cpd.error_count != 0) ? 1 : 0);
+   return((cpd.error_count != 0) ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
 
@@ -626,28 +635,40 @@ static void process_source_list(const char *source_list,
    }
 
    char linebuf[256];
-   int  argc;
-   char *args[3];
+   char *fname;
    int  line = 0;
-   int  idx;
+   int  len;
 
    while (fgets(linebuf, sizeof(linebuf), p_file) != NULL)
    {
       line++;
-      argc = Args::SplitLine(linebuf, args, ARRAY_SIZE(args));
-
-      LOG_FMT(LFILELIST, "%3d]", line);
-      for (idx = 0; idx < argc; idx++)
+      fname = linebuf;
+      len   = strlen(fname);
+      while ((len > 0) && unc_isspace(*fname))
       {
-         LOG_FMT(LFILELIST, " [%s]", args[idx]);
+         fname++;
+         len--;
+      }
+      while ((len > 0) && unc_isspace(fname[len - 1]))
+      {
+         len--;
+      }
+      fname[len] = 0;
+      while (len-- > 0)
+      {
+         if (fname[len] == '\\')
+         {
+            fname[len] = '/';
+         }
       }
-      LOG_FMT(LFILELIST, "\n");
 
-      if ((argc == 1) && (*args[0] != '#'))
+      LOG_FMT(LFILELIST, "%3d] %s\n", line, fname);
+
+      if (fname[0] != '#')
       {
          char outbuf[1024];
-         do_source_file(args[0],
-                        make_output_filename(outbuf, sizeof(outbuf), args[0], prefix, suffix),
+         do_source_file(fname,
+                        make_output_filename(outbuf, sizeof(outbuf), fname, prefix, suffix),
                         NULL, no_backup, keep_mtime);
       }
    }
@@ -677,7 +698,7 @@ static bool read_stdin(file_mem& fm)
 
    /* Copy the raw data from the deque to the vector */
    fm.raw.insert(fm.raw.end(), dq.begin(), dq.end());
-   return decode_unicode(fm.raw, fm.data, fm.enc, fm.bom);
+   return(decode_unicode(fm.raw, fm.data, fm.enc, fm.bom));
 }
 
 
@@ -951,11 +972,11 @@ static void do_source_file(const char *filename_in,
                            bool       no_backup,
                            bool       keep_mtime)
 {
-   FILE       *pfout;
-   bool       did_open    = false;
-   bool       need_backup = false;
-   file_mem   fm;
-   string     filename_tmp;
+   FILE     *pfout;
+   bool     did_open    = false;
+   bool     need_backup = false;
+   file_mem fm;
+   string   filename_tmp;
 
    /* Do some simple language detection based on the filename extension */
    if (!cpd.lang_forced || (cpd.lang_flags == 0))
@@ -1036,6 +1057,7 @@ static void do_source_file(const char *filename_in,
          else
          {
 #ifdef WIN32
+
             /* windows can't rename a file if the target exists, so delete it
              * first. This may cause data loss if the tmp file gets deleted
              * or can't be renamed.
@@ -1304,6 +1326,8 @@ static void uncrustify_start(const deque<int>& data)
       add_file_footer();
    }
 
+   annotations_newlines();
+
    /**
     * Change certain token types based on simple sequence.
     * Example: change '[' + ']' to '[]'
@@ -1439,10 +1463,15 @@ static void uncrustify_file(const file_mem& fm, FILE *pfout,
       do_parens();
 
       /**
-       * Insert line breaks as needed
+       * Modify line breaks as needed
        */
-      //do_newlines();
-      int old_changes;
+      bool first = true;
+      int  old_changes;
+
+      if (cpd.settings[UO_nl_remove_extra_newlines].n == 2)
+      {
+         newlines_remove_newlines();
+      }
       cpd.pass_count = 3;
       do
       {
@@ -1451,7 +1480,7 @@ static void uncrustify_file(const file_mem& fm, FILE *pfout,
          LOG_FMT(LNEWLINE, "Newline loop start: %d\n", cpd.changes);
 
          newlines_cleanup_dup();
-         newlines_cleanup_braces();
+         newlines_cleanup_braces(first);
          if (cpd.settings[UO_nl_after_multiline_comment].b)
          {
             newline_after_multiline_comment();
@@ -1481,6 +1510,7 @@ static void uncrustify_file(const file_mem& fm, FILE *pfout,
          if (cpd.settings[UO_pos_arith].tp != TP_IGNORE)
          {
             newlines_chunk_pos(CT_ARITH, cpd.settings[UO_pos_arith].tp);
+            newlines_chunk_pos(CT_CARET, cpd.settings[UO_pos_arith].tp);
          }
          newlines_class_colon_pos();
          if (cpd.settings[UO_nl_squeeze_ifdef].b)
@@ -1490,6 +1520,7 @@ static void uncrustify_file(const file_mem& fm, FILE *pfout,
          do_blank_lines();
          newlines_eat_start_end();
          newlines_cleanup_dup();
+         first = false;
       } while ((old_changes != cpd.changes) && (cpd.pass_count-- > 0));
 
       mark_comments();
@@ -1551,25 +1582,29 @@ static void uncrustify_file(const file_mem& fm, FILE *pfout,
       }
 
       /**
-       * Aligning everything else and reindent
+       * Align everything else, reindent and break at code_width
        */
-      align_all();
-      indent_text();
-
-      if (cpd.settings[UO_code_width].n > 0)
+      first          = true;
+      cpd.pass_count = 3;
+      do
       {
-         cpd.pass_count = 3;
-         do
+         align_all();
+         indent_text();
+         old_changes = cpd.changes;
+         if (cpd.settings[UO_code_width].n > 0)
          {
-            old_changes = cpd.changes;
+            LOG_FMT(LNEWLINE, "Code_width loop start: %d\n", cpd.changes);
+
             do_code_width();
-            if (old_changes != cpd.changes)
+            if ((old_changes != cpd.changes) && first)
             {
-               align_all();
-               indent_text();
+               /* retry line breaks caused by splitting 1-liners */
+               newlines_cleanup_braces(false);
+               newlines_insert_blank_lines();
+               first = false;
             }
-         } while ((old_changes != cpd.changes) && (cpd.pass_count-- > 0));
-      }
+         }
+      } while ((old_changes != cpd.changes) && (cpd.pass_count-- > 0));
 
       /**
        * And finally, align the backslash newline stuff
@@ -1700,6 +1735,10 @@ struct file_lang languages[] =
    { ".hpp",  "",     LANG_CPP           },
    { ".hxx",  "",     LANG_CPP           },
    { ".cc",   "",     LANG_CPP           },
+   { ".cp",   "",     LANG_CPP           },
+   { ".C",    "",     LANG_CPP           },
+   { ".CPP",  "",     LANG_CPP           },
+   { ".c++",  "",     LANG_CPP           },
    { ".di",   "",     LANG_D             },
    { ".m",    "OC",   LANG_OC            },
    { ".mm",   "OC+",  LANG_OC | LANG_CPP },
